//==========================================
// CLEANREG - Matt Pietrek 1996
// Microsoft Systems Journal, September 1996
// FILE: CLEANREG.CPP
//==========================================
#include <windows.h>
#include <string.h>
#include <stdio.h>
#include <malloc.h>
#include <tchar.h>
#pragma hdrstop
#include "cleanreg.h"

// From CLNREGUI.CPP
void AddItemToUI(LPTSTR pszRegPath, LPTSTR pszValueName, LPTSTR pszFilename);

BOOL g_fBeRealistic = TRUE; // Eliminate unlikely (but possible) filenames

//
// Checks to see if the file exist on a fixed drive, and if not,
// adds it to the user interface
//
void CheckForExistence(LPTSTR pszRegPath, LPTSTR pszValueName, LPTSTR pszFile)
{
    // See if the file is on a floppy or CD-ROM drive.  Don't tell about, if so
    TCHAR szDriveRoot[4];
    _tcsncpy( szDriveRoot, pszFile, 3 );
    szDriveRoot[3] = 0;

    UINT driveType = GetDriveType(szDriveRoot);
    if ( (driveType != DRIVE_FIXED) && (driveType != DRIVE_REMOTE) )
        return;
    
    // A quick'n'easy way to see if a file exists.  Bail out if so.
    if ( 0xFFFFFFFF != GetFileAttributes( pszFile ) )
        return;

    AddItemToUI( pszRegPath, pszValueName, pszFile );   // Not found! Add to UI
}

//
// Given a string as input, this routine does its best to try and extract
// a filename portion in the beginning.  A "filename" means that there's
// a ":\" near the beginning.  The function handles filenames in quotes.
// If the g_fBeRealistic variable is TRUE, it assumes that characters like
// '/' and '-' aren't part of the filename, although they're technically legal.
//
BOOL CheckForFilename( LPTSTR psz, LPTSTR szOutput, UINT cbOutput )
{
    if ( lstrlen(psz) <= 2 )        // toss out strings that are too short
        return FALSE;

    BOOL fBeginQuote = FALSE;
    LPTSTR pszStartFilename = psz;
    if ( *pszStartFilename == '"' ) // Does the string start with a quote?
    {                               // If so, adjust accordingly.  We'll
        fBeginQuote = TRUE;         // assume that there must be an end quote
        pszStartFilename++;         // for it to be a legal filename.
    }
    
    LPTSTR pszEndName;      // Pointer to one char past the end of the name
    
    if ( fBeginQuote )  // The string started with quotes.  Find the other one
    {
        pszEndName = _tcschr( pszStartFilename, '"' );
        if ( !pszEndName )
            return FALSE;
        
        pszEndName--;   // Strip off spaces at the end
        while ( (pszEndName > pszStartFilename) && (*pszEndName == ' ') )
            pszEndName--;

        pszEndName++;
    }
    else    // String didn't start with a quote
    {
        // Look for the first non-legal filename character
        pszEndName = _tcspbrk( psz, TEXT("<>|/\"") );

        if ( pszEndName )   // We found a non-legal character, so the
        {                   // filename must end before this
            // Now strip off any whitespace
            pszEndName--;
            while ( (pszEndName > pszStartFilename) && (*pszEndName == ' '))
                pszEndName--;
            
            pszEndName++;
        }
        else
            pszEndName = psz + lstrlen(psz);    // No non-legal chars found
    }

    // At this point, pszStartFilename points to the beginning of the possible
    // filename, and pszEndName points 1 character past the last character
    // of the possible filename.
        
    if ( pszEndName - pszStartFilename < 2 )    // Are the start and end far
        return FALSE;                           // enough apart to be legal?

    // Check to see if the 2nd and 3rd characters are ":\"
    if ( 0 != _tcsnicmp( pszStartFilename+1, TEXT(":\\"), 2 ) )
        return FALSE;

    if ( g_fBeRealistic )   // If set, toss out unlikely characters
    {
        LPTSTR pszSearch = _tcspbrk( pszStartFilename, TEXT("^/-%,;") );
        
        // Did we find something, and was it before what we're considering
        // the end of the filename to be?
        if ( pszSearch && (pszSearch < pszEndName) )
        {
            // Now strip off any whitespace
            pszSearch--;
            while ( (pszSearch > pszStartFilename) && (*pszSearch == ' ') )
                pszSearch--;

            pszEndName = pszSearch+1;
        }
    }

    // Make sure the string we found won't overflow the input buffer
    unsigned nameLen = pszEndName - pszStartFilename; 
    if ( nameLen >= cbOutput )
        return FALSE;
    
    // copy the filename string to the input buffer
    memcpy( szOutput, pszStartFilename, nameLen * sizeof(TCHAR) );
    szOutput[ nameLen ] = 0;    // Null terminate it!
    
    return TRUE;
}

//
// Routine that enumerates through all the values in a registry, looking
// for REG_SZ's that begin with filenames.  Then, the routine recurses
// to iterate through its child nodes.
//
void ScanRegNode( HKEY hKey, LPTSTR pszRegPath )
{
    DWORD regEnumIndex = 0;

    while ( TRUE )
    {
        TCHAR szValueName[ MAX_PATH ];
        DWORD cbValueNameSize = SIZEOFARRAY(szValueName);
        TCHAR szValue[ MAX_PATH ];
        DWORD cbValueSize = SIZEOFARRAY(szValue);
        DWORD type;
        LONG result;
        
        result = RegEnumValue(  hKey, regEnumIndex++,
                                szValueName, &cbValueNameSize,
                                0, &type, (LPBYTE)szValue, &cbValueSize );
                            
        if ( ERROR_NO_MORE_ITEMS == result )
            break;

        if ( REG_SZ != type )
            continue;

        TCHAR szFilename[MAX_PATH];
        
        if ( CheckForFilename(  szValue,
                                szFilename, SIZEOFARRAY(szFilename)) )
            CheckForExistence( pszRegPath, szValueName, szFilename );
    }

    // ================ Now iterate through all the child nodes ===============

    regEnumIndex = 0;
    
    while ( TRUE )
    {
        TCHAR szSubkey[ MAX_PATH ];
        DWORD cbSubkeySize = SIZEOFARRAY(szSubkey);
        LONG result;
        
        result = RegEnumKeyEx(  hKey, regEnumIndex++, szSubkey, &cbSubkeySize,
                                0, 0, 0, 0 );

        if ( ERROR_NO_MORE_ITEMS == result )
            break;

        if ( 0 != result )  // Some other error?  Ignore and continue
            continue;

        HKEY hSubkey;
        
        result = RegOpenKeyEx( hKey, szSubkey, 0, KEY_READ, &hSubkey );
                            
        if ( ERROR_SUCCESS != result )
            continue;

        // Create a fully qualified subkey name to pass to ScanRegNode
        LPTSTR pszRegPathSubkey
            = new TCHAR[lstrlen(pszRegPath) + cbSubkeySize + 2];
        wsprintf( pszRegPathSubkey, TEXT("%s\\%s"), pszRegPath, szSubkey );

        // Recurse to find any child keys
        ScanRegNode( hSubkey, pszRegPathSubkey );

        delete []pszRegPathSubkey;

        RegCloseKey( hSubkey );
    }
}

void SearchRegistryForMissingFiles(void)
{
    // Start the scan at the top level.  Note that we don't do keys that
    // are mirrored elsewhere (e.g., HKEY_CLASSES_ROOT), or that we
    // shouldn't muck with (HKEY_USERS)
    ScanRegNode( HKEY_CURRENT_USER, TEXT("\\HKEY_CURRENT_USER") );
    ScanRegNode( HKEY_LOCAL_MACHINE, TEXT("\\HKEY_LOCAL_MACHINE") );
}

//
// Recursive routine to delete a key and all of its subkeys.  We can't just
// use RegDeleteKey, since NT doesn't do recursive deletions.  (For once,
// Win95 did something better than NT!).  Based on PSS article Q142491
//
LONG RecursiveRegDeleteKey( HKEY hKey, LPTSTR pszSubkey )
{
    HKEY hSubkey;

    LONG result = RegOpenKeyEx( hKey, pszSubkey, 0,
                                KEY_ENUMERATE_SUB_KEYS | DELETE, &hSubkey );

    while ( ERROR_SUCCESS == result )
    {
        TCHAR szSubkey[ MAX_PATH ];
        DWORD cbSubkeySize = SIZEOFARRAY(szSubkey);
        
        result = RegEnumKeyEx(  hSubkey, 0, szSubkey, &cbSubkeySize,
                                0, 0, 0, 0);

        if ( ERROR_NO_MORE_ITEMS == result )
            break;

        if ( ERROR_SUCCESS != result )
            return result;

        result = RecursiveRegDeleteKey( hSubkey, szSubkey );
    }

    RegCloseKey( hSubkey );
    result = RegDeleteKey( hKey, pszSubkey );

    return result;
}

// Data structures for mapping string representation of root registry keys
// back to their binary values

struct REG_ROOT_NAME
{
    LPTSTR pszName;
    HKEY   hKey;
};

REG_ROOT_NAME RegRootKeys[] =
{
{ TEXT("HKEY_CLASSES_ROOT"),    HKEY_CLASSES_ROOT },
{ TEXT("HKEY_CURRENT_USER"),    HKEY_CURRENT_USER },
{ TEXT("HKEY_LOCAL_MACHINE"),   HKEY_LOCAL_MACHINE },
{ TEXT("HKEY_USERS"),           HKEY_USERS },
{ TEXT("HKEY_PERFORMANCE_DATA"),HKEY_PERFORMANCE_DATA },
{ TEXT("HKEY_CURRENT_CONFIG"),  HKEY_CURRENT_CONFIG },
{ TEXT("HKEY_DYN_DATA"),        HKEY_DYN_DATA },
};

//
// Given a registry path, deletes the specified key and everything below it.
// Expects a string in the form \Key1\Key2\Key3\
// A value can be specified via the form \[value: xxx]\
// Thus, the path \Key1\Key2\[value: xxx]\string can be passed
//
BOOL DeleteRegistryPath( LPTSTR _pszRegPath )
{
    if ( !_pszRegPath || *_pszRegPath != '\\' )
        return FALSE;
    
    // Make a copy of the string so that we can munge it
    LPTSTR pszRegPath = _tcsdup( _pszRegPath );
    if ( !pszRegPath )
        return FALSE;

    //
    // First, break out the root portion of the path (e.g., "\HKEY_CLASSES\")
    //
    LPTSTR pszRootKey, pszRootKeyEnd, pszSubkey=0;
    pszRootKey = pszRegPath+1;
    pszRootKeyEnd = _tcschr( pszRootKey, '\\' );
    if ( !pszRootKeyEnd )
    {
        free( pszRegPath );
        return FALSE;
    }
    else
    {
        *pszRootKeyEnd = 0;
        pszSubkey = pszRootKeyEnd + 1;
    }

    // Look up the predefined HKEY
    HKEY hKeyRoot = 0;
    for ( unsigned i = 0; i < SIZEOFARRAY(RegRootKeys); i++ )
        if ( 0 == _tcscmp( pszRootKey, RegRootKeys[i].pszName ) )
        {
            hKeyRoot = RegRootKeys[i].hKey;
            break;
        }
        
    if ( 0 == hKeyRoot )    // Huh??? Not found.  This shouldn't happen
    {
        free( pszRegPath );
        return FALSE;       
    }

    //
    // Next, determine if there's a value included in the reg path.  If so,
    // isolate it and strip it off.
    //
        
    LPTSTR pszValue = _tcsstr( pszSubkey, TEXT("\\[value: ") );
    
    if ( pszValue )     // Did we find a "\[value: xxx]" somewhere???
    {
        *pszValue = 0;  // Null terminate pszRegPath before the "\[value: "
        pszValue += 9;  // Advance the pointer past "\[value: "
        
        // Now strip off the terminating ']'
        LPTSTR pszEndValue = _tcschr( pszValue, ']' );
        if ( pszEndValue )
            *pszEndValue = 0;
        
        // Check for the special case of "<default>", which is the default value
        if ( 0 == _tcscmp( pszValue, TEXT("<default>") ) )
            pszValue = TEXT("");
    }

    //
    // At this point, we have a predefined root key, a subkey string, and an
    // optional value string.  If we have a value string, delete the value
    // from the registry.  Otherwise, delete the key.
    //
        
    BOOL fSuccess = FALSE;      // Assume the worst...
        
    if ( pszValue )             // We're just deleting a value (RegDeleteValue)
    {
        HKEY hSubkey;
        if ( ERROR_SUCCESS == RegOpenKeyEx( hKeyRoot, pszSubkey,
                                            0, KEY_ALL_ACCESS, &hSubkey ) )
        {
            if ( ERROR_SUCCESS == RegDeleteValue( hSubkey, pszValue ) )
                fSuccess = TRUE;
            RegCloseKey( hSubkey );
        }
    }
    else    // We're deleting an entire key
    {
        // Split off the last node, and null terminate the prior nodes
        LPTSTR pszSubkey2 = _tcsrchr( pszSubkey, '\\' );
        *pszSubkey2++ = 0;

        HKEY hSubkey;
        if ( ERROR_SUCCESS == RegOpenKeyEx( hKeyRoot, pszSubkey,
                                            0, KEY_ENUMERATE_SUB_KEYS | DELETE,
                                            &hSubkey ) )
        {
            if ( ERROR_SUCCESS == RecursiveRegDeleteKey( hSubkey, pszSubkey2) )
                fSuccess = TRUE;
            RegCloseKey( hSubkey );
        }
    }
    
    free( pszRegPath );
    return fSuccess;
}
