//==========================================
// CLEANREG - Matt Pietrek 1996
// Microsoft Systems Journal, September 1996
// FILE: CLNREGUI.CPP
//==========================================
#include <windows.h>
#include <commctrl.h>
#include <tchar.h>
#pragma hdrstop
#include "clnregui.h"
#include "cleanreg.h"

// Helper function prototypes
void Handle_WM_COMMAND(HWND hDlg, WPARAM wParam, LPARAM lParam);
void Handle_WM_INITDIALOG(HWND hDlg);
void Handle_WM_CLOSE( HWND hDlg );
BOOL Handle_WM_NOTIFY( HWND hDlg, WPARAM wParam, LPARAM lParam );
void Handle_ListBox_Cmd(HWND hDlg, WORD wNotifyCode, WORD wID, HWND hwndCtl);
BOOL Handle_TreeView_Notify( HWND hDlg, int idCtrl, LPNMHDR pnmh );
BOOL CALLBACK CleanRegDlgProc( HWND, UINT, WPARAM, LPARAM );
HTREEITEM TVAppendItem( HWND hWnd, HTREEITEM hTi, LPTSTR psz );
BOOL GetRegPathForHTI(HWND hWnd, HTREEITEM hTi, PTSTR pszPath, unsigned cbPath);
void DeleteRegPathAfterQuerying( HWND hWnd, LPTSTR pszRegPath );
void RefreshViews(void);            

HWND g_hWndTree=0;      // Global variables (Yes, the horror!) for speed
HWND g_hWndList = 0;
HTREEITEM g_hTiRoot;
BOOL g_fRefreshAfterDelete = TRUE;

TCHAR g_szRootName[] = TEXT("My Registry");
TCHAR g_szCopyright[] = TEXT("CleanReg - Matt Pietrek, 1996 for MSJ");

TCHAR g_szHelpAbout[] =
TEXT("CleanReg searches the registry for filenames that aren't present on ")
TEXT("a fixed or network drive.  These files are listed in the top listbox. ")
TEXT("When a file is selected, the bottom treeview expands to show the ")
TEXT("registry path where the filename was found.\r\n\r\n")
TEXT("Nodes in the treeview can be selected. All keys and values below and ")
TEXT("including the node can be deleted via the <delete> key.  Use this ")
TEXT("option with care!!!\r\n\r\n")
TEXT("CleanReg is from the September 1996 issue of the Microsoft Systems ")
TEXT("Journal.  Please refer to that issue (which will be on the MSDN ")
TEXT("CD-ROM) for all documentation questions.");

int PASCAL WinMain( HANDLE hInstance, HANDLE hPrevInstance,
                    PSTR lpszCmdLine, int nCmdShow )
{
    InitCommonControls();    // Gotta do this for treeview controls

    // Bring up the user interface (A dialog box?  What a surprise!)
    DialogBox(  hInstance, MAKEINTRESOURCE(IDD_DIALOG_CLEANREG),
                0, (DLGPROC)CleanRegDlgProc );
    return 0;
}

//=================== Start of message handling routines ====================

BOOL CALLBACK CleanRegDlgProc(HWND hDlg,UINT msg,WPARAM wParam,LPARAM lParam)
{
    switch ( msg )
    {
        case WM_COMMAND:
            Handle_WM_COMMAND( hDlg, wParam, lParam ); return TRUE;
        case WM_INITDIALOG:
            Handle_WM_INITDIALOG( hDlg ); return TRUE;
        case WM_CLOSE:
            Handle_WM_CLOSE( hDlg ); break;
        case WM_NOTIFY:
            return Handle_WM_NOTIFY( hDlg, wParam, lParam );
    }
    return FALSE;
}

void Handle_WM_COMMAND( HWND hDlg, WPARAM wParam, LPARAM lParam )
{
    WORD wNotifyCode = HIWORD(wParam);
    WORD wID = LOWORD(wParam);
    HWND hwndCtl = (HWND)lParam;

    switch ( wID )
    {
        case IDC_LIST:
            Handle_ListBox_Cmd( hDlg, wNotifyCode, wID, hwndCtl ); break;
        case IDC_ABOUT:
            MessageBox( hDlg, g_szHelpAbout, g_szCopyright, MB_OK ); break;
        case IDC_CHECK_SANE:
            if ( BN_CLICKED == wNotifyCode )
            {
                g_fBeRealistic = (BOOL)IsDlgButtonChecked(hDlg,IDC_CHECK_SANE);
                RefreshViews(); // Valid file list may have changed, so update
            }
        case IDC_CHECK_UPDATE:
            if ( BN_CLICKED == wNotifyCode )
                g_fRefreshAfterDelete =
                    (BOOL)IsDlgButtonChecked( hDlg,IDC_CHECK_UPDATE );
            break;
    }
}

void Handle_WM_INITDIALOG(HWND hDlg)
{
    g_hWndTree = GetDlgItem( hDlg, IDC_TREE );  // Save control HWND's in
    g_hWndList = GetDlgItem( hDlg, IDC_LIST );  // global variables for speed

    CheckDlgButton( hDlg, IDC_CHECK_SANE,   // Set checkboxes appropriately
                    g_fBeRealistic ? BST_CHECKED : BST_UNCHECKED );
    CheckDlgButton( hDlg, IDC_CHECK_UPDATE,
                    g_fRefreshAfterDelete ? BST_CHECKED : BST_UNCHECKED );

    RefreshViews();   // Fill treeviews with initial values
}

void Handle_WM_CLOSE( HWND hDlg )
{
    EndDialog(hDlg, 0);
}

// Tree controls like to use WM_NOTIFY, rather than WM_COMMAND
BOOL Handle_WM_NOTIFY( HWND hDlg, WPARAM wParam, LPARAM lParam )
{
    int idCtrl = wParam;
    LPNMHDR pnmh = (LPNMHDR)lParam;
    
    if ( IDC_TREE == idCtrl )
        return Handle_TreeView_Notify( hDlg, idCtrl, pnmh );

    return FALSE;
}

void Handle_ListBox_Cmd(HWND hDlg, WORD wNotifyCode, WORD wID, HWND hwndCtl)
{
    // If the file listbox selection changes, expand the appropriate tree item
    if ( (g_hWndList == hwndCtl) && (wNotifyCode == LBN_SELCHANGE) )
    {
        // Figure out which listbox item was selected
        LRESULT lbIndex = SendMessage( hwndCtl, LB_GETCURSEL, 0, 0 );
        if ( LB_ERR == lbIndex )
            return;

        // Now get the treeview HTREEITEM that we associated with this entry
        HTREEITEM hTi= 
            (HTREEITEM)SendMessage(hwndCtl, LB_GETITEMDATA, (WPARAM)lbIndex, 0);

        // Set treeview selection, based on the HTREEITEM we just retrieved
        TreeView_Select( g_hWndTree, hTi, TVGN_CARET );
	
		SetFocus( g_hWndTree );	// Set focus to the treeview for convenience
    }
}

BOOL Handle_TreeView_Notify( HWND hDlg, int idCtrl, LPNMHDR pnmh )
{
    if ( TVN_KEYDOWN == pnmh->code )    // We want keystrokes
    {
         TV_KEYDOWN * ptvkd = (TV_KEYDOWN *)pnmh;
         if ( VK_DELETE == ptvkd->wVKey )   // Was it the <delete> key?
         {
             TCHAR szRegPath[1024];     // Hope this is big enough!
             
             if ( GetRegPathForHTI(  g_hWndTree,
                                     TreeView_GetSelection(g_hWndTree),
                                     szRegPath,
                                     sizeof(szRegPath)/sizeof(TCHAR) ) )
             {
                 DeleteRegPathAfterQuerying( hDlg, szRegPath );
             }
         }
    }
    
    return TRUE;
}

//====================== Start of helper routines ===========================

//
// Do a complete update of the UI, based on rescanning the registry
//
void RefreshViews(void)
{
    HCURSOR hCursOld = SetCursor( LoadCursor(0, IDC_WAIT) );    // hourglass!

    // Clear out the listbox and tell it to not redraw after each addition
    SendMessage( g_hWndList, LB_RESETCONTENT, 0, 0 );
    SendMessage( g_hWndList, WM_SETREDRAW, FALSE, 0 );
    
    // Clear out the treeview, and add the root treeview node
    TreeView_DeleteAllItems( g_hWndTree );
    g_hTiRoot = TVAppendItem( g_hWndTree, TVI_ROOT, g_szRootName );
    
    SearchRegistryForMissingFiles();    // Get down to the real work

    SendMessage( g_hWndList, WM_SETREDRAW, TRUE, 0 );   // Let listbox redraw
    SetCursor( hCursOld );
}

BOOL TVGetItemText( HWND hWnd, HTREEITEM hTi,
                    LPTSTR pszText, unsigned cbText )
{
    TV_ITEM tvi;
    tvi.mask = TVIF_TEXT;
    tvi.hItem = hTi;
    tvi.pszText = pszText;
    tvi.cchTextMax = cbText;
    return TreeView_GetItem( hWnd, &tvi );
}

// Append a new child node to the specified HTREEITEM
HTREEITEM TVAppendItem( HWND hWnd, HTREEITEM hTi, LPTSTR psz )
{
    // A helper function that appends a string to the end of the list
    // of subnodes under a specified node (hTi).

    TV_INSERTSTRUCT tvis;
    memset( &tvis, 0, sizeof(tvis) );
    
    tvis.hParent = hTi;
    tvis.hInsertAfter = TVI_LAST;
    tvis.item.mask = TVIF_TEXT;
    tvis.item.pszText = psz;
    return TreeView_InsertItem( hWnd, &tvis );  
}

//
// Given an HTREEITEM, return a '\' delimite path to it (e.g., "\a\b\c\d")
//
BOOL GetRegPathForHTI(HWND hWnd, HTREEITEM hTi, LPTSTR pszPath, unsigned cbPath)
{
    TCHAR szNode[MAX_PATH];
    unsigned cbThisNode =0, cbSoFar = 0;
    
    while ( hTi )
    {
        if ( !TVGetItemText( hWnd, hTi, szNode, sizeof(szNode)/sizeof(TCHAR)) )
            return FALSE;
        
        cbThisNode = lstrlen(szNode) + 1;
        cbSoFar += cbThisNode;
        if ( cbSoFar >= cbPath )
            return FALSE;
        
        memmove( pszPath + cbThisNode, pszPath, cbSoFar * sizeof(TCHAR) );
        pszPath[0] = '\\';
        memcpy( pszPath+1, szNode, (cbThisNode-1) * sizeof(TCHAR) );
        
        hTi = TreeView_GetParent( hWnd, hTi );
    }
    
    pszPath[cbSoFar] = 0;   // Null terminate the string
        
    return TRUE;
}

//
// Given a '\' path (e.g., "\a\b\c\d", find its HTREEITEM in the treeview.
// If the node doesn't exist, build it!  See also GetRegPathForHTI()
//
HTREEITEM GetHTIForRegPath( HWND hWnd, HTREEITEM hTi, LPTSTR pszRegPath )
{
    //
    // Fill in the szKey variable with the name of the current registry
    // node being searched for.  The nodes are separated by '\'s.  The
    // szKey variable is a string representing one registry key node, and
    // without any '\'s
    //
    TCHAR szKey[MAX_PATH];
    LPTSTR pszSlash = _tcschr( pszRegPath, '\\' );
    if ( pszSlash )
    {
        unsigned len = pszSlash - pszRegPath;
        memcpy( szKey, pszRegPath, len * sizeof(TCHAR) );
        szKey[len] = 0;
    }
    else    // No slash found, so it's the last node found.  Just copy it.
        _tcscpy( szKey, pszRegPath );

    //
    // Now look through the child nodes of the HTREEITEM that we're currently
    // working on.  If we find it, but there's still more nodes in pszRegpath,
    // recurse to find the next node in the registry path.  
    //
    HTREEITEM hTiScan;
    hTiScan = TreeView_GetNextItem( hWnd, hTi, TVGN_CHILD );
    while ( hTiScan )
    {
        TCHAR szScanItemText[MAX_PATH];
        
        if ( TVGetItemText( hWnd, hTiScan, szScanItemText,
                            sizeof(szScanItemText)/sizeof(TCHAR) ) )
        {
            if ( 0 == _tcscmp( szKey, szScanItemText ) )
            {
                if ( pszSlash )
                    return GetHTIForRegPath( hWnd, hTiScan, pszSlash+1 );
                else
                    return hTiScan;
            }
        }
            
        hTiScan = TreeView_GetNextItem( hWnd, hTiScan, TVGN_NEXT );
    }

    //
    // If we get here, we didn't find the item, so create it
    //
    HTREEITEM hTiNew = TVAppendItem( hWnd, hTi, szKey );
    if ( !hTiNew )
        return 0;

    //
    // If there's still nodes left in pszRegpath, recurse so that they can
    // be created.  Otherwise, return the HTREEITEM of the new treeview item.
    //
    if ( pszSlash )
        return GetHTIForRegPath( hWnd, hTiNew, pszSlash+1 );
    else
        return hTiNew;
}

//
// Called by CheckForExistence in CLEANREG.CPP when a non-existent file is
// found in the registry.  Add the filename to the listbox and treeview.
//
void AddItemToUI( LPTSTR pszRegPath, LPTSTR pszValueName, LPTSTR pszFilename )
{
    // Get an HTREEITEM for the registry path, making the nodes if necessary
    HTREEITEM hTi = GetHTIForRegPath(   g_hWndTree,
                                        g_hTiRoot,
                                        pszRegPath+1 );

    // Make a new node representing the value portion (in []'s), and
    // add it underneath the actual keys.  If the value name is empty, then
    // it's the default value, so call it "<default>"
    TCHAR szValueNode[MAX_PATH*2];
    wsprintf( szValueNode, TEXT("[value: %s]"),
                *pszValueName ? pszValueName : TEXT("<default>") );
    hTi = TVAppendItem( g_hWndTree, hTi, szValueNode );
    
    // now put the filename portion of the string under its value node
    hTi = TVAppendItem( g_hWndTree, hTi, pszFilename );

    // Add the filename to the listbox, and associate the HTREEITEM of the
    // filename as it appears in the treeview.
    DWORD lbIndex;
    lbIndex = SendMessage( g_hWndList, LB_ADDSTRING, 0, (LPARAM) pszFilename );
    SendMessage( g_hWndList, LB_SETITEMDATA, (WPARAM)lbIndex, (LPARAM)hTi );
}

//
// If the user hits the <delete> key on an item in the treeview, this routine
// querys the user to make sure it's OK to delete the registry entry.  If
// the user says "OK", it calls DeleteRegistryPath() in CLEANREG.CPP
//
void DeleteRegPathAfterQuerying( HWND hWnd, LPTSTR pszRegPath )
{
    if ( *pszRegPath != '\\' )
        return;
    
    // Make sure that pszRegPath starts with "My Registry" (or whatever we've
    // decided to call the root node in the treeview control.
    if ( 0 != _tcsncmp(pszRegPath+1, g_szRootName, SIZEOFARRAY(g_szRootName)-1) )
        return;

    // Advance past the first node in pszRegPath.  It's just there for the
    // treeview control, and not part of the actual registry path.
    pszRegPath += SIZEOFARRAY(g_szRootName);
        
    if ( IDOK == MessageBox( hWnd, pszRegPath, TEXT("Deleting:"),
                            MB_OKCANCEL | MB_ICONWARNING | MB_DEFBUTTON2 ) )
    {
        if ( DeleteRegistryPath( pszRegPath ) )
        {
            if ( g_fRefreshAfterDelete )
                RefreshViews();
        }
        else
            MessageBox( hWnd, TEXT("Error in deleting"), 0, MB_OK );
    }
}
