/*************************************************************
Module name: DllStub.c
Written by:  Jeffrey Richter
Purpose:     Stub DLL that contains only a DllMain function 
             and function forwarders to functions in DllWork.
*************************************************************/


#define STRICT
#include <Windows.h>
#include "..\DllWork\DllWork.h"


//////////////////////////////////////////////////////////////


// Function forwarders to functions in DllWork
#pragma comment(linker, "/export:SomeFunc=DllWork.SomeFunc")


//////////////////////////////////////////////////////////////


BOOL WINAPI DllMain (HINSTANCE hinstDll, 
   DWORD fdwReason, LPVOID p) {

   switch (fdwReason) {
   case DLL_PROCESS_ATTACH:
      DebugBox("Dll loading", "DllStub");
      break;

   case DLL_THREAD_ATTACH:
   case DLL_THREAD_DETACH:
      break;

   case DLL_PROCESS_DETACH:
      // When this DLL is detached, tell the DllWork DLL
      // to shut itself down cleanly
      ShutdownLibrary();
      DebugBox("Dll unloading", "DllStub");
      break;
   }

   return(TRUE);  // Successful initialization
}


///////////////////////// End of File ////////////////////////
