////////////////////////////////////////////////////////////////////////////////
// $Header:   S:\3x\src\ntkm\ntdevpp\sampdrv\adapter.cpv   1.0   24 Jun 1996 16:24:24   JHUCKINS  $
// 
// INTEL Corporation Proprietary Information
// 
// This listing is supplied under the terms of a license agreement
// with INTEL Corporation and may not be copied nor disclosed except
// in accordance with the terms of that agreement.
// 
// Copyright (c) 1996 Intel Corporation. All rights reserved.
// 
// $Revision:   1.0  $
// $Date:   24 Jun 1996 16:24:24  $
// $Author:   JHUCKINS  $
// 
// Log at end of file.
// 
// Module Name:	adapter.cpp
// Author:		Jeff Huckins
// Abstract:	CAdapterObject Implementation
// Environment:	Windows NT 3.51/4.0, MS Visual C++ 2.x
// Notes:
// 
////////////////////////////////////////////////////////////////////////////////

// disable some warnings
#pragma warning( disable: 4699 )	// Note: Creating precompiled header
#pragma warning( disable: 4201 )	// nameless struct/union
#pragma warning( disable: 4514 )	// unreferenced inline function has been removed
#pragma warning( disable: 4705 )	// statement has no effect
#pragma warning( disable: 4711 )	// function 'function' selected for automatic inline expansion

#include <idrivrpp.h>

//////////////////////////////////////////////////////////////////////////////////
// ctor/dtor
//
CAdapterObject::CAdapterObject( 
	IN ULONG CommonBufferLength /* = 0 */,
	IN ULONG CommonBufferAlignment /* = 0 */,
	IN BOOLEAN bCacheEnabled /* = FALSE */ )
	:	m_AdapterObject( NULL ),
		m_NumberOfMapRegisters( 0 ),
		m_CommonBuffer( NULL ),
		m_CommonBufferAligned( NULL ),
		m_CommonBufferLogicalAddress( 0 ),
		m_CommonBufferLogicalAddressAligned( 0 ),
		m_CommonBufferLength( CommonBufferLength ),
		m_CommonBufferAlignment( CommonBufferAlignment ),
		m_bCacheEnabled( bCacheEnabled )
{
	return;
}

CAdapterObject::~CAdapterObject()
{
	return;
}

////////////////////////////////////////////////////////////////////////////////
// initialization routines
//
NTSTATUS CAdapterObject::Initialize(
	IN PUNICODE_STRING RegistryPath,
	IN ULONG CommonBufferLength,
	IN ULONG CommonBufferAlignment,
	IN BOOLEAN bCacheEnabled,
	IN PDEVICE_DESCRIPTION DeviceDescription )
{
	// set member variables
	m_CommonBufferLength = CommonBufferLength;
	m_CommonBufferAlignment = CommonBufferAlignment;
	m_bCacheEnabled = bCacheEnabled;

	// get adapter that matches DeviceDescription
	if( FALSE == GetAdapter( DeviceDescription ) )
	{
		return STATUS_INSUFFICIENT_RESOURCES;
	}

	// are we allocating any data
	if( 0 < CommonBufferLength )
	{
		// allocate the common buffer
		if( FALSE == AllocateCommonBuffer() )
		{
			return STATUS_INSUFFICIENT_RESOURCES;
		}
	}

	return OnInitialize( RegistryPath );
}

NTSTATUS CAdapterObject::Initialize( 
	IN PUNICODE_STRING RegistryPath, 
	IN PDEVICE_DESCRIPTION DeviceDescription )
{
	if( FALSE == GetAdapter( DeviceDescription ) )
	{
		return STATUS_INSUFFICIENT_RESOURCES;
	}

	return OnInitialize( RegistryPath );
}

////////////////////////////////////////////////////////////////////////////////
// cleanup routine
//
VOID CAdapterObject::Unload( void )
{
	// give derived classes a crack at cleanup
	OnUnload();

	// free the common buffer if there is one
	if( NULL != m_CommonBuffer )
	{
		FreeCommonBuffer();
	}

	return;
}

////////////////////////////////////////////////////////////////////////////////
// gets an adapter object
//
BOOLEAN CAdapterObject::GetAdapter( IN PDEVICE_DESCRIPTION DeviceDescription )
{
	ASSERT( NULL != DeviceDescription );

	if( NULL != m_AdapterObject )
	{
		// @@@ why return false... we have an adapter object
		// @@@ (we just don't know its device description)
		return FALSE;
	}

	m_AdapterObject = HalGetAdapter( DeviceDescription, &m_NumberOfMapRegisters );
	return BOOLEAN( m_AdapterObject != NULL );
}

////////////////////////////////////////////////////////////////////////////////
// allocates the common buffer
//
BOOLEAN CAdapterObject::AllocateCommonBuffer( void )
{
	ASSERT( PAGE_SIZE >= HalGetDmaAlignmentRequirement() );
	ASSERT( 0 == (PAGE_SIZE % HalGetDmaAlignmentRequirement()) );

	// determine if buffer fix up for alignment reasons is required
	ULONG BufferSize;
	BOOLEAN FixUp = FALSE;
	if( PAGE_SIZE < m_CommonBufferAlignment )
	{
		// buffer alignment is greater than a page, we need to fix up
		BufferSize = (m_CommonBufferLength > m_CommonBufferAlignment) ?
			m_CommonBufferLength + m_CommonBufferAlignment : m_CommonBufferAlignment * 2;
		FixUp = TRUE;
	}
	else
	{
		// buffer alignment is less than or equal to a page
		// we need to allocate at least a page to ensure an aligned buffer
		BufferSize = (m_CommonBufferLength >= PAGE_SIZE) ? m_CommonBufferLength : PAGE_SIZE;
	}

	// allocate the buffer
	if( NULL !=	(m_CommonBuffer = HalAllocateCommonBuffer( 
		m_AdapterObject, BufferSize, m_CommonBufferLogicalAddress, m_bCacheEnabled )) )
	{
		ASSERT( m_CommonBuffer & (PAGE_SIZE - 1) &&
			m_CommonBufferLogicalAddress & (PAGE_SIZE - 1) );
		m_CommonBufferAligned = m_CommonBuffer;
		m_CommonBufferLogicalAddressAligned = m_CommonBufferLogicalAddress;

		if( TRUE == FixUp )
		{
			// compute offset
			ULONG BufferOffset = m_CommonBufferAlignment -
				(m_CommonBufferLogicalAddress.LowPart % m_CommonBufferAlignment);

			// add offset to buffers
			m_CommonBufferAligned = PVOID( ULONG( m_CommonBufferAligned ) + BufferOffset );
			m_CommonBufferLogicalAddressAligned.LowPart += BufferOffset;
		}

		return TRUE;
	}
	else
	{
		return FALSE;
	}
}

////////////////////////////////////////////////////////////////////////////////
// frees the common buffer
//
VOID CAdapterObject::FreeCommonBuffer( void )
{
	ASSERT( NULL != m_AdapterObject );
	ASSERT( NULL != m_CommonBuffer );

	// returns void
	HalFreeCommonBuffer(
		m_AdapterObject,
		m_CommonBufferLength,
		m_CommonBufferLogicalAddress,
		m_CommonBuffer,
		m_bCacheEnabled );

	// reset data members
	m_CommonBuffer = NULL;
	m_CommonBufferAligned = NULL;
	m_CommonBufferLogicalAddress = 0;
	m_CommonBufferLogicalAddressAligned = 0;

	return;
}

////////////////////////////////////////////////////////////////////////////////
// overridables
//
NTSTATUS CAdapterObject::OnInitialize( IN PUNICODE_STRING /* = RegistryPath */ )
{
	return STATUS_SUCCESS;
}

VOID CAdapterObject::OnUnload( void )
{
	return;
}

////////////////////////////////////////////////////////////////////////////////
// $Log:   S:\3x\src\ntkm\ntdevpp\sampdrv\adapter.cpv  $
// 
//    Rev 1.0   24 Jun 1996 16:24:24   JHUCKINS
// Initial revision.
// 
//    Rev 1.8   03 Jun 1996 07:45:10   JHUCKINS
// Added Author to file header
// 
//    Rev 1.7   31 May 1996 15:48:42   JDELLIOT
// added robustness features (ASSERT, TRACE, validation of params)
// cleanup
// restructure of some interfaces
// 
//    Rev 1.6   17 May 1996 10:57:56   JDELLIOT
// modified alignment code to align based on physical address vs. virtual
// address (this should make no difference - it just seems conceptually more
// correct)
// 
//    Rev 1.5   13 May 1996 13:26:26   JDELLIOT
// added common buffer alignment support
// 
//    Rev 1.4   04 Apr 1996 09:45:20   JHUCKINS
// Added IN and OUT macros to formal argument lists
// 
//    Rev 1.3   03 Apr 1996 07:29:02   JHUCKINS
// include idrivrpp.h instead of ntdevpp.h so that global operator new and
// delete are defined for all
// 
//    Rev 1.2   27 Mar 1996 14:12:04   JDELLIOT
// added PVCS keywords and support for warning level 4
//

