////////////////////////////////////////////////////////////////////////////////
// $Header:   S:\3x\src\ntkm\ntdevpp\sampdrv\cppx.cpv   1.0   24 Jun 1996 16:24:22   JHUCKINS  $
// 
// INTEL Corporation Proprietary Information
// 
// This listing is supplied under the terms of a license agreement
// with INTEL Corporation and may not be copied nor disclosed except
// in accordance with the terms of that agreement.
// 
// Copyright (c) 1996 Intel Corporation. All rights reserved.
// 
// $Revision:   1.0  $
// $Date:   24 Jun 1996 16:24:22  $
// $Author:   JHUCKINS  $
// 
// Log at end of file.
// 
// Module Name:	cppx.cpp
// Author:		Jeff Huckins
// Abstract:	C++ init and shutdown implementation
// Environment:	Windows NT 3.51/4.0, MS Visual C++ 2.x
// Notes:
// 
////////////////////////////////////////////////////////////////////////////////

// disable some warnings
#pragma warning( disable: 4699 )	// Note: Creating precompiled header
#pragma warning( disable: 4201 )	// nameless struct/union
#pragma warning( disable: 4514 )	// unreferenced inline function has been removed
#pragma warning( disable: 4711 )	// function 'function' selected for automatic inline expansion

extern "C"
{
	#include <ntddk.h>
}

//////////////////////////////////////////////////////////////////////////////
// C++ Startup Code
//
// Note, the following code has special knowledge of the symbols that are
// present in the standard libc.lib (__xc_a and __xc_z).  Those symbols and
// the data they represent are NOT present in the NT Kernel-Mode Driver library
// ntoskrnl.lib.
//////////////////////////////////////////////////////////////////////////////

typedef void (__cdecl *PF)( void );

extern "C" PF __xc_a[], __xc_z[];	// These are in LIBC.LIB's data segs

static void CallCPPInitializers( PF* pfBegin, PF* pfEnd )
{
	// walk the table of function pointers from the bottom up, until the end
	// is encountered.  Do not skip the first entry.  The initial value of
	// pfbegin points to the first valid entry.  Do not try to execute what
	// pfend points to.  Only entries before pfend are valid.
	while( pfBegin < pfEnd )
	{
		// if current table entry is non-NULL, call thru it.
		if( NULL != *pfBegin )
		{
			(**pfBegin)();
		}

		++pfBegin;
	}
}

////////////////////////////////////////////////////////////
// Typedef for atexit handlers
//
typedef void (__cdecl* PATEXIT_HANDLER)( void );

////////////////////////////////////////////////////////////
// Table of handlers
//
#define MAX_ATEXIT_HANDLERS 32	// Defined by the language for atexit()
static PATEXIT_HANDLER _AtExitHandlers[ MAX_ATEXIT_HANDLERS ];
static int _iNumHandlers = 0;

////////////////////////////////////////////////////////////////////////////////
// Initializes the C++ run-time subsystem by calling global constructors.
//
extern "C" BOOLEAN cppInit( void )
{
	// We start with no handlers...
	_iNumHandlers = 0;

	// NULL-out our table of function pointers
	int i;
	for( i = 0; i < MAX_ATEXIT_HANDLERS; i++ )
	{
		_AtExitHandlers[ i ] = 0;
	}

	// Call C++ initializers
	CallCPPInitializers( __xc_a, __xc_z );

	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////
// Calls global destructors.
//    
extern "C" void cppShutdown( void )
{
	// The docs for atexit() state that this list will be walked
	// in LIFO order, so do that...
	int iHandler;
	for( iHandler = _iNumHandlers - 1; iHandler >= 0; iHandler-- )
	{
		PATEXIT_HANDLER pHandler = _AtExitHandlers[ iHandler ];
		ASSERT( NULL != pHandler );
		(*pHandler)();
	}

	// Make sure we don't run this code again
	_iNumHandlers = 0;
}

////////////////////////////////////////////////////////////////////////////////
// atexit()        Definition taken from the language guide.
//
// The atexit function is passed the address of a function (pHandler) to be
// called when the program terminates normally.  Successive calls to atexit
// create a register of functions that are executed in LIFO (last-in-first-out)
// order.  No more than 32 functions can be registered with atexit or _onexit.
// The functions passed to atexit cannot take parameters.
//
// Return value
// atexit will return 0 if successful, or a nonzero value if an error occurs
// (e.g., if there are already 32 exit functions defined).
//
extern "C" int __cdecl atexit( PATEXIT_HANDLER pHandler )
{
	// Bounds check and validate the function pointer
	if( (MAX_ATEXIT_HANDLERS > _iNumHandlers) && (NULL != pHandler) ) 
	{
		// Store it...
		_AtExitHandlers[ _iNumHandlers ] = pHandler;

		// On to the next slot...
		_iNumHandlers++;

		// Success
		return 0;
	}

	// Failed; either the table is full or the function pointer was NULL
	return -1;
}

////////////////////////////////////////////////////////////////////////////////
// $Log:   S:\3x\src\ntkm\ntdevpp\sampdrv\cppx.cpv  $
// 
//    Rev 1.0   24 Jun 1996 16:24:22   JHUCKINS
// Initial revision.
// 
//    Rev 1.4   03 Jun 1996 07:47:02   JHUCKINS
// Added Author to file header
// 
//    Rev 1.3   31 May 1996 15:49:36   JDELLIOT
// added robustness features (ASSERT, TRACE, validation of params)
// cleanup
// restructure of some interfaces
// 
//    Rev 1.2   27 Mar 1996 14:12:40   JDELLIOT
// added PVCS keywords and support for warning level 4
//

