////////////////////////////////////////////////////////////////////////////////
// $Header:   S:\3x\src\ntkm\ntdevpp\sampdrv\devqueue.cpv   1.0   24 Jun 1996 16:24:24   JHUCKINS  $
//
// INTEL Corporation Proprietary Information
//
// This listing is supplied under the terms of a license agreement
// with INTEL Corporation and may not be copied nor disclosed except
// in accordance with the terms of that agreement.
//
// Copyright (c) 1996 Intel Corporation. All rights reserved.
//
// $Revision:   1.0  $
// $Date:   24 Jun 1996 16:24:24  $
// $Author:   JHUCKINS  $
//
// Log at end of file.
//
// Module Name:		devqueue.cpp
// Abstract:		CDeviceQueue Implementation
// Environment:		Windows NT 3.51/4.0, MS Visual C++ 2.x
// Notes:
//
////////////////////////////////////////////////////////////////////////////////

// disable some warnings
#pragma warning( disable: 4699 )	// Note: Creating precompiled header
#pragma warning( disable: 4201 )	// nameless struct/union
#pragma warning( disable: 4514 )	// unreferenced inline function has been removed
#pragma warning( disable: 4711 )	// function 'function' selected for automatic inline expansion

#include <idrivrpp.h>
#include "debug.h"

////////////////////////////////////////////////////////////////////////////////
// ctor/dtor
//
CDeviceQueue::CDeviceQueue( void )
{
	// this cannot fail
	::KeInitializeDeviceQueue( &m_DeviceQueue );
}

CDeviceQueue::~CDeviceQueue( void )
{
	ASSERT( NULL == Remove() );
}

BOOLEAN CDeviceQueue::Insert( IN CDeviceQueueEntry* DeviceQueueEntry )
{
	ASSERT( NULL != DeviceQueueEntry );
	ASSERT_IRQL_LE( DISPATCH_LEVEL );
	
	// raise IRQL if necessary
	KIRQL Irql = DISPATCH_LEVEL;
	if( DISPATCH_LEVEL != ::KeGetCurrentIrql() )
	{
		KeRaiseIrql( DISPATCH_LEVEL, &Irql );
	}

	BOOLEAN Result = ::KeInsertDeviceQueue( &m_DeviceQueue, DeviceQueueEntry );

	// lower IRQL if necessary
	if( DISPATCH_LEVEL != Irql )
	{
		KeLowerIrql( Irql );
	}

	return Result;
}

BOOLEAN CDeviceQueue::Insert( IN CDeviceQueueEntry* DeviceQueueEntry, IN ULONG SortKey )
{
	ASSERT( NULL != DeviceQueueEntry );
	ASSERT_IRQL_LE( DISPATCH_LEVEL );
	
	// raise IRQL if necessary
	KIRQL Irql = DISPATCH_LEVEL;
	if( DISPATCH_LEVEL != ::KeGetCurrentIrql() )
	{
		KeRaiseIrql( DISPATCH_LEVEL, &Irql );
	}

	BOOLEAN Result = ::KeInsertByKeyDeviceQueue( &m_DeviceQueue, DeviceQueueEntry, SortKey );

	// lower IRQL if necessary
	if( DISPATCH_LEVEL != Irql )
	{
		KeLowerIrql( Irql );
	}

	return Result;
}

BOOLEAN CDeviceQueue::InsertData( IN PVOID Data )
{
	CDeviceQueueEntry* DeviceQueueEntry = new CDeviceQueueEntry( Data );
	if( NULL != DeviceQueueEntry )
	{
		if( FALSE == Insert( DeviceQueueEntry ) )
		{
			delete DeviceQueueEntry;
			return FALSE;
    	}
		else
		{
			return TRUE;
		}
	}
	else
	{
		return FALSE;
	}
}

BOOLEAN CDeviceQueue::InsertData( IN PVOID Data, IN ULONG SortKey )
{
	CDeviceQueueEntry* DeviceQueueEntry = new CDeviceQueueEntry( Data );
	if( NULL != DeviceQueueEntry )
	{
		if( FALSE == Insert( DeviceQueueEntry, SortKey ) )
		{
			delete DeviceQueueEntry;
			return FALSE;
    	}
		else
		{
			return TRUE;
		}
	}
	else
	{
		return FALSE;
	}
}

CDeviceQueueEntry* CDeviceQueue::Remove( void )
{
	ASSERT_IRQL_LE( DISPATCH_LEVEL );
	
	// raise IRQL if necessary
	KIRQL Irql = DISPATCH_LEVEL;
	if( DISPATCH_LEVEL != ::KeGetCurrentIrql() )
	{
		KeRaiseIrql( DISPATCH_LEVEL, &Irql );
	}

	CDeviceQueueEntry* Result = (CDeviceQueueEntry*) ::KeRemoveDeviceQueue( &m_DeviceQueue );

	// lower IRQL if necessary
	if( DISPATCH_LEVEL != Irql )
	{
		KeLowerIrql( Irql );
	}

	return Result;
}

CDeviceQueueEntry* CDeviceQueue::Remove( IN ULONG SortKey )
{
	ASSERT_IRQL_LE( DISPATCH_LEVEL );
	
	// raise IRQL if necessary
	KIRQL Irql = DISPATCH_LEVEL;
	if( DISPATCH_LEVEL != ::KeGetCurrentIrql() )
	{
		KeRaiseIrql( DISPATCH_LEVEL, &Irql );
	}

	CDeviceQueueEntry* Result = (CDeviceQueueEntry*) ::KeRemoveByKeyDeviceQueue( &m_DeviceQueue, SortKey );

	// lower IRQL if necessary
	if( DISPATCH_LEVEL != Irql )
	{
		KeLowerIrql( Irql );
	}

	return Result;
}

PVOID CDeviceQueue::RemoveData( void )
{
	CDeviceQueueEntry* DeviceQueueEntry = Remove();
	if( NULL != DeviceQueueEntry )
	{
		PVOID Data = DeviceQueueEntry->GetData();
		delete DeviceQueueEntry;
		return Data;
	}
	else
	{
		return NULL;
	}
}

PVOID CDeviceQueue::RemoveData( IN ULONG SortKey )
{
	CDeviceQueueEntry* DeviceQueueEntry = Remove( SortKey );
	if( NULL != DeviceQueueEntry )
	{
		PVOID Data = DeviceQueueEntry->GetData();
		delete DeviceQueueEntry;
		return Data;
	}
	else
	{
		return NULL;
	}
}

BOOLEAN CDeviceQueue::RemoveEntry( IN CDeviceQueueEntry* DeviceQueueEntry )
{
	ASSERT( NULL != DeviceQueueEntry );
	ASSERT_IRQL_LE( DISPATCH_LEVEL );
	
	// raise IRQL if necessary
	KIRQL Irql = DISPATCH_LEVEL;
	if( DISPATCH_LEVEL != ::KeGetCurrentIrql() )
	{
		KeRaiseIrql( DISPATCH_LEVEL, &Irql );
	}

	BOOLEAN Result = ::KeRemoveEntryDeviceQueue( &m_DeviceQueue, DeviceQueueEntry );

	// lower IRQL if necessary
	if( DISPATCH_LEVEL != Irql )
	{
		KeLowerIrql( Irql );
	}

	return Result;
}

///////////////////////////////////////////////////////////////////////////////
// $Log:   S:\3x\src\ntkm\ntdevpp\sampdrv\devqueue.cpv  $
// 
//    Rev 1.0   24 Jun 1996 16:24:24   JHUCKINS
// Initial revision.
// 
//    Rev 1.2   31 May 1996 15:49:50   JDELLIOT
// added robustness features (ASSERT, TRACE, validation of params)
// cleanup
// restructure of some interfaces
// 
//    Rev 1.1   08 May 1996 15:28:46   JDELLIOT
// added xxxData routines
// 
//    Rev 1.0   07 May 1996 14:53:56   JDELLIOT
// Initial revision.
// 
// 
