////////////////////////////////////////////////////////////////////////////////
// $Header:   S:\3x\src\ntkm\ntdevpp\sampdrv\driver.h_v   1.0   24 Jun 1996 16:24:30   JHUCKINS  $
// 
// INTEL Corporation Proprietary Information
// 
// This listing is supplied under the terms of a license agreement
// with INTEL Corporation and may not be copied nor disclosed except
// in accordance with the terms of that agreement.
// 
// Copyright (c) 1996 Intel Corporation. All rights reserved.
// 
// $Revision:   1.0  $
// $Date:   24 Jun 1996 16:24:30  $
// $Author:   JHUCKINS  $
// 
// Log at end of file.
// 
// Module Name:	driver.h
// Abstract:	CDriverObject Interface
// Environment:	Windows NT 3.51/4.0, MS Visual C++ 4.x
// Notes:
//		CDriverObject encapsulates the DRIVER_OBJECT data structure.
//
//		IMPORTANT:
//			A CDriverObject class object MUST be statically instantiated by the
//			user.  Dynamic instantiation is not supported. Each driver object
//			represents the image of a loaded kernel-mode driver. A pointer to
//			the driver object is an input parameter to a driver's DriverEntry
//			and optional Reinitialize routines and to its Unload routine, if
//			any.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef driver_h
#define driver_h

#include "device.h"

class CDriverObject 
{
public:
	// ctor/dtor
	CDriverObject( PCWSTR DriverClassName = NULL );
	virtual ~CDriverObject();

	// Simply returns NULL because dynamic instantiation of an
	// object of this class is not supported.
	PVOID operator new( size_t size );

	// Cast operator for PDRIVER_OBJECT.
	operator PDRIVER_OBJECT();

	VOID SetDriverObject( PDRIVER_OBJECT DriverObject );

	// called from within DriverEntry routine. Must call
	// IoRegisterDriverReinitialization passing this pointer as Context
	// argument.  Then must call virtual OnInitialize() member.
	NTSTATUS Initialize( PUNICODE_STRING RegistryPath );

	// called from within DriverUnload() routine.  This routine MUST:
	//		1. Call m_DeviceObject->Unload()
	//	    2. Call virtual OnUnload()
	static VOID Unload( PDRIVER_OBJECT DriverObject );

	// Driver Entry routine exported to IOManager.
	static NTSTATUS DriverEntry( PDRIVER_OBJECT DriverObject, PUNICODE_STRING RegistryPath );

	// Calls IoAssignResources().
	NTSTATUS AssignResources(
		IN PUNICODE_STRING RegistryPath, 
		IN PIO_RESOURCE_REQUIREMENTS_LIST RequestedResources, 
		OUT PCM_RESOURCE_LIST* AllocatedResources );

	NTSTATUS AssignSlotResources(
		IN PUNICODE_STRING RegistryPath, 
		IN INTERFACE_TYPE BusType,  
		IN ULONG BusNumber, 
		IN ULONG SlotNumber, 
		OUT PCM_RESOURCE_LIST* AllocatedResources );

	// calls IoReportResourceUsage().
	NTSTATUS ReportResourceUsage(
		IN PCM_RESOURCE_LIST DriverList, 
		IN ULONG DriverListSize, 
		IN BOOLEAN bOverRideConflict, 
		OUT PBOOLEAN bConflictDetected );

	// set in constructor
	PUNICODE_STRING GetClassName( VOID );

	PUNICODE_STRING GetHardwareDatabase( VOID );

protected:
	// called from Unload() member, gives derived classes an opportunity to
	// cleanup before UnloadDevices is called (e.g. if driver supports a
	// DEVICE_OBJECT that is not a CDeviceObject)
	virtual VOID OnUnload( void );

	// called by Initialize() routine, gives derived classes an opportunity to
	// perform additional initialization (e.g. if driver supports a
	// DEVICE_OBJECT that is not a CDeviceObject)
	virtual NTSTATUS OnInitialize( PUNICODE_STRING RegistryPath );

	// calls IoRegisterDriverReinitialization().
	VOID RegisterDriverReinitialization(
		PDRIVER_REINITIALIZE DriverReinitializationRoutine,
		PVOID Context );

	// CreateDevices can create device objects during
	// CDriverObject::Initialize().  Following is an example of how a derived
	// class can create CDeviceObject objects:
	//		NTSTATUS CMyDriverObject::CreateDevices( void )
	//		{
	//			NTSTATUS status;
	//			new( *this, DeviceName, DeviceType, DeviceCharacteristics,
	//				bExclusive, status ) CMyDeviceObject( this );
	//			return status;
	//		}
	virtual NTSTATUS CreateDevices( void ) = 0;

	// setup as StartIo callback for DRIVER_OBJECT, calls OnStartIo()
	static VOID StartIo( IN PDEVICE_OBJECT DeviceObject, IN PIRP Irp );

	// calls OnReadStartIo(), OnWriteStartIo(), or OnStartIo members of
	// CDeviceObject.  Override to perform custom StartIo handling (e.g.
	// if driver supports a DEVICE_OBJECT that is not a CDeviceObject)
	virtual VOID OnStartIo( IN PDEVICE_OBJECT DeviceObject, IN PIRP Irp );

	// dispatch handler, calls OnIrpMjXxx
	static NTSTATUS IrpMjXxx( IN PDEVICE_OBJECT DeviceObject, IN PIRP Irp );

	// overridable dispatch handler. Override to perform custom StartIo
	// handling (e.g. if driver supports a DEVICE_OBJECT that is not a
	// CDeviceObject)
	virtual NTSTATUS OnIrpMjXxx( IN PDEVICE_OBJECT DeviceObject, IN PIRP Irp );

private:
	// methods
	NTSTATUS InitDevices( PUNICODE_STRING RegistryPath );
	VOID UnloadDevices( VOID );

	// members
	PDRIVER_OBJECT m_DriverObject;

	// the only driver object
	static CDriverObject* s_DriverObject;

	//	Set in constructor
	UNICODE_STRING m_ClassName;
};

////////////////////////////////////////////////////////////////////////////////
// inlines
//
inline CDriverObject::operator PDRIVER_OBJECT()
{
	return m_DriverObject;
}

inline VOID CDriverObject::SetDriverObject( IN PDRIVER_OBJECT DriverObject )
{
	ASSERT( NULL == m_DriverObject );
	ASSERT( NULL != DriverObject );

	m_DriverObject = DriverObject;
	return;
}

inline PUNICODE_STRING CDriverObject::GetClassName( void )
{
	return &m_ClassName;
}

inline PUNICODE_STRING CDriverObject::GetHardwareDatabase( void )
{
	return m_DriverObject->HardwareDatabase;
}

#endif

/*******************************************************************************
 * $Log:   S:\3x\src\ntkm\ntdevpp\sampdrv\driver.h_v  $
 * 
 *    Rev 1.0   24 Jun 1996 16:24:30   JHUCKINS
 * Initial revision.
 * 
 *    Rev 1.15   11 Jun 1996 11:31:08   JDELLIOT
 * moved dispatch table and StartIo setup to OnInitialize
 * moved IoRegisterShutdownNotification to InitDevices
 * added support for all know Irps
 * added default support for unknown Irps
 * 
 *    Rev 1.14   03 Jun 1996 11:44:14   JDELLIOT
 * changed dispatch handlers to a single handler
 * 
 *    Rev 1.13   31 May 1996 15:49:56   JDELLIOT
 * added robustness features (ASSERT, TRACE, validation of params)
 * cleanup
 * restructure of some interfaces
 * 
 *    Rev 1.12   22 Apr 1996 10:17:24   JHUCKINS
 * Moved driver entry-points from CDeviceObject to CDriverObject so that the
 * CDriverObject can have an opportunity to filter dispatches to its 
 * CDeviceObjects.
 * 
 *    Rev 1.11   04 Apr 1996 09:45:18   JHUCKINS
 * Added IN and OUT macros to formal argument lists
 * 
 *    Rev 1.10   03 Apr 1996 16:44:40   JHUCKINS
 * Use IoReportResourceUsage() instead of IoAssignResources OR HalAssignSlot
 * Resources() to claim resources.
 * 
 *    Rev 1.9   02 Apr 1996 16:32:02   JHUCKINS
 * checkpoint
 * 
 *    Rev 1.8   01 Apr 1996 15:58:02   JHUCKINS
 * Made GetRegistryPath() public for access by CDeviceObject classes.
 * 
 *    Rev 1.7   27 Mar 1996 14:12:56   JDELLIOT
 * added PVCS keywords and support for warning level 4
 *
 */

