////////////////////////////////////////////////////////////////////////////////
// $Header:   S:\3x\src\ntkm\ntdevpp\sampdrv\intrpt.cpv   1.0   24 Jun 1996 16:24:26   JHUCKINS  $
// 
// INTEL Corporation Proprietary Information
// 
// This listing is supplied under the terms of a license agreement
// with INTEL Corporation and may not be copied nor disclosed except
// in accordance with the terms of that agreement.
// 
// Copyright (c) 1996 Intel Corporation. All rights reserved.
// 
// $Revision:   1.0  $
// $Date:   24 Jun 1996 16:24:26  $
// $Author:   JHUCKINS  $
// 
// Log at end of file.
// 
// Module Name:	intrpt.cpp
// Author:		Jeff Huckins
// Abstract:	CInterrupt Object Implementation
// Environment:	Windows NT 3.51/4.0, MS Visual C++ 2.x
// Notes:
// 
////////////////////////////////////////////////////////////////////////////////

// disable some warnings
#pragma warning( disable: 4699 )	// Note: Creating precompiled header
#pragma warning( disable: 4201 )	// nameless struct/union
#pragma warning( disable: 4514 )	// unreferenced inline function has been removed
#pragma warning( disable: 4711 )	// function 'function' selected for automatic inline expansion

#include <idrivrpp.h>
#include "intrpt.h"
#include "device.h"

////////////////////////////////////////////////////////////////////////////////
// ctor/dtor
//
CInterruptObject::CInterruptObject( IN CDeviceObject* DeviceObject /* = NULL */ )
	:	m_Vector( 0 ),
		m_Irql( 0 ),
		m_Affinity( 0 ),
		m_InterruptObject( NULL ),
		m_DeviceObject( DeviceObject )
{
	return;
}

CInterruptObject::~CInterruptObject()
{
	return;
}

NTSTATUS CInterruptObject::Initialize(
	IN PUNICODE_STRING RegistryPath, 
	IN INTERFACE_TYPE InterfaceType, 
	IN ULONG BusNumber, 
	IN ULONG BusInterruptLevel, 
	IN ULONG BusInterruptVector)
{
	ASSERT_IRQL_EQ( PASSIVE_LEVEL );

	m_Vector = HalGetInterruptVector( 
		InterfaceType, 
		BusNumber, 
		BusInterruptLevel, 
		BusInterruptVector, 
		&m_Irql, 
		&m_Affinity );

	return OnInitialize( RegistryPath );
}

NTSTATUS CInterruptObject::OnInitialize( IN PUNICODE_STRING /* RegistryPath */ )
{
	return STATUS_SUCCESS;
}

NTSTATUS CInterruptObject::Connect(
	IN PKSPIN_LOCK SpinLock, 
	IN KIRQL SynchronizeIrql, 
	IN KINTERRUPT_MODE InterruptMode, 
	IN BOOLEAN ShareVector, 
	IN BOOLEAN FloatingSave )
{
	ASSERT_IRQL_LE( PASSIVE_LEVEL );

	// Initialize the spinlock for the caller.
	if( NULL != SpinLock )
	{
		KeInitializeSpinLock( SpinLock );
	}

	return IoConnectInterrupt( 
		&m_InterruptObject,
		Isr,
		this,
		SpinLock,
		m_Vector,
		m_Irql,
		SynchronizeIrql,
		InterruptMode,
		ShareVector,
		m_Affinity,
		FloatingSave );
}

VOID CInterruptObject::Disconnect( void )
{
	ASSERT_IRQL_LE( PASSIVE_LEVEL );

	if( NULL != m_InterruptObject )
	{
		IoDisconnectInterrupt( m_InterruptObject );
		m_InterruptObject = NULL;
	}

	return;
}

BOOLEAN CInterruptObject::SynchronizeRoutineForRead(
	IN PVOID SynchronizedContext )
{
	CDeviceObject* pDevice = (CDeviceObject*) SynchronizedContext;
	ASSERT( NULL != pDevice );

	return pDevice->OnSynchronizeExecutionForRead();
}

BOOLEAN CInterruptObject::SynchronizeRoutineForWrite(
	IN PVOID SynchronizedContext )
{
	CDeviceObject* pDevice = (CDeviceObject*) SynchronizedContext;
	ASSERT( NULL != pDevice );

	return pDevice->OnSynchronizeExecutionForWrite();
}

BOOLEAN CInterruptObject::Isr(
	IN PKINTERRUPT /* Interrupt */,
	IN PVOID ServiceContext )
{
	CInterruptObject* Interrupt = (CInterruptObject*) ServiceContext;
	ASSERT( NULL != Interrupt );

	return Interrupt->OnInterrupt();
}

BOOLEAN CInterruptObject::OnInterrupt( void )
{
	if( NULL != m_DeviceObject )
	{
		return m_DeviceObject->OnInterrupt();
	}
	else
	{
		return FALSE;
	}
}

/////////////////////////////////////////////////////////////////////////////
// $Log:   S:\3x\src\ntkm\ntdevpp\sampdrv\intrpt.cpv  $
// 
//    Rev 1.0   24 Jun 1996 16:24:26   JHUCKINS
// Initial revision.
// 
//    Rev 1.10   11 Jun 1996 11:31:38   JDELLIOT
// added support for CInterruptObject objects that are not connected to
// CDeviceObject objects
// 
//    Rev 1.9   03 Jun 1996 07:47:04   JHUCKINS
// Added Author to file header
// 
//    Rev 1.8   31 May 1996 15:50:06   JDELLIOT
// added robustness features (ASSERT, TRACE, validation of params)
// cleanup
// restructure of some interfaces
// 
//    Rev 1.7   07 May 1996 09:32:34   JHUCKINS
// Added assertions for PASSIVE_LEVEL on Connect() and Disconnect()
// 
//    Rev 1.6   04 Apr 1996 09:45:22   JHUCKINS
// Added IN and OUT macros to formal argument lists
// 
//    Rev 1.5   03 Apr 1996 07:29:40   JHUCKINS
// include idrivrpp.h instead of ntdevpp.h so that global operator new and
// delete are defined for all
// 
//    Rev 1.4   27 Mar 1996 14:49:08   JHUCKINS
// Added SynchronizeExecution() members for Read and Write
// 
//    Rev 1.3   27 Mar 1996 14:13:16   JDELLIOT
// added PVCS keywords and support for warning level 4
// 
// 
