////////////////////////////////////////////////////////////////////////////////
// $Header:   S:\3x\src\ntkm\ntdevpp\sampdrv\object.h_v   1.0   24 Jun 1996 16:24:36   JHUCKINS  $
// 
// INTEL Corporation Proprietary Information
// 
// This listing is supplied under the terms of a license agreement
// with INTEL Corporation and may not be copied nor disclosed except
// in accordance with the terms of that agreement.
// 
// Copyright (c) 1996 Intel Corporation. All rights reserved.
// 
// $Revision:   1.0  $
// $Date:   24 Jun 1996 16:24:36  $
// $Author:   JHUCKINS  $
// 
// Log at end of file.
// 
// Module Name:	object.h
// Author:		Jeff Huckins
// Abstract:	CObject, CNamedObject and CFileObject Interfaces
// Environment:	Windows NT 3.51/4.0, MS Visual C++ 4.x
// Notes:
// 
////////////////////////////////////////////////////////////////////////////////

#ifndef object_h
#define object_h

////////////////////////////////////////////////////////////////////////////////
// CObject Interface
////////////////////////////////////////////////////////////////////////////////
class CObject
{
public:
	// ctor/dtor
	CObject();
	virtual ~CObject();

	NTSTATUS Close( VOID );

	virtual NTSTATUS GetObjectPtr( IN PVOID& Object );

	// to be called after object creation
	virtual BOOLEAN IsValid( VOID );

	operator HANDLE() const;

protected:
	// methods
	NTSTATUS ReferenceObjectByHandle(
		IN ACCESS_MASK DesiredAccess,
		IN POBJECT_TYPE ObjectType = NULL,
		IN KPROCESSOR_MODE AccessMode = KernelMode,
		IN POBJECT_HANDLE_INFORMATION = NULL,
		IN BOOLEAN CloseHandle = TRUE,
		OUT NTSTATUS* CloseStatus = NULL );

	// members
	HANDLE	m_Handle;
	PVOID	m_ObjectPtr;
};

////////////////////////////////////////////////////////////////////////////////
// inlines
//
inline CObject::CObject()
	:	m_Handle( NULL ),
		m_ObjectPtr( NULL )
{
	return;
}

inline NTSTATUS CObject::Close( void )
{
	NTSTATUS Status = STATUS_SUCCESS;
	if( NULL != m_Handle )
	{
		Status = ZwClose( m_Handle );
		m_Handle = NULL;
	}

	return Status;
}

inline CObject::operator HANDLE() const
{
	return m_Handle;
}

////////////////////////////////////////////////////////////////////////////////
// CNamedObject Interface
////////////////////////////////////////////////////////////////////////////////
class CNamedObject : public CObject
{
public:
	BOOLEAN SetName( IN PCWSTR Name, IN KIRQL MaxIrql = PASSIVE_LEVEL );
	BOOLEAN SetName( IN CUnicodeString& Name, IN KIRQL MaxIrql = PASSIVE_LEVEL );
	CUnicodeString& GetName( VOID );

protected:
	CUnicodeString	m_Name;
};

////////////////////////////////////////////////////////////////////////////////
// inlines
//
inline CUnicodeString& CNamedObject::GetName( VOID )
{
	return m_Name;
}

inline BOOLEAN CNamedObject::SetName(
	IN PCWSTR Name,
	IN KIRQL MaxIrql /* = PASSIVE_LEVEL */ )
{
	return m_Name.Copy( Name, MaxIrql > PASSIVE_LEVEL ? NonPagedPool : PagedPool );
}

inline BOOLEAN CNamedObject::SetName(
	IN CUnicodeString& Name,
	IN KIRQL MaxIrql /* = PASSIVE_LEVEL */ )
{
	return m_Name.Copy( Name, MaxIrql > PASSIVE_LEVEL ? NonPagedPool : PagedPool );
}

////////////////////////////////////////////////////////////////////////////////
// CFileObject Interface
////////////////////////////////////////////////////////////////////////////////
class CFileObject : public CNamedObject
{
public:
	NTSTATUS Create(
		IN ACCESS_MASK DesiredAccess,
		IN POBJECT_ATTRIBUTES ObjectAttributes,
		OUT PIO_STATUS_BLOCK IoStatusBlock,
		IN PLARGE_INTEGER AllocationSize,
		IN ULONG FileAttributes,
		IN ULONG ShareAccess,
		IN ULONG CreateDisposition,
		IN ULONG CreateOptions );

	// For easy way to create a RootDirectory object
	NTSTATUS Create(
		IN ACCESS_MASK DesiredAccess,
		OUT PIO_STATUS_BLOCK IoStatusBlock,
		IN ULONG FileAttributes = FILE_ATTRIBUTE_NORMAL,
		IN ULONG ShareAccess = FILE_SHARE_READ | FILE_SHARE_WRITE,
		IN ULONG CreateDisposition = FILE_OPEN_IF,
		IN ULONG CreateOptions = FILE_DIRECTORY_FILE );
};

////////////////////////////////////////////////////////////////////////////////
// inlines
//
inline NTSTATUS CFileObject::Create(
	IN ACCESS_MASK DesiredAccess,
	IN POBJECT_ATTRIBUTES ObjectAttributes,
	OUT PIO_STATUS_BLOCK IoStatusBlock,
	IN PLARGE_INTEGER AllocationSize,
	IN ULONG FileAttributes,
	IN ULONG ShareAccess,
	IN ULONG CreateDisposition,
	IN ULONG CreateOptions )
{
	ASSERT( NULL == m_Handle );
	ASSERT( NULL != ObjectAttributes );
	ASSERT( NULL != IoStatusBlock );

	return ZwCreateFile(
		&m_Handle,
		DesiredAccess,
		ObjectAttributes,
		IoStatusBlock,
		AllocationSize,
		FileAttributes,
		ShareAccess,
		CreateDisposition,
		CreateOptions,
		NULL,	// EaBuffer
		0 );	// EaLength
}

#endif object_h

/*******************************************************************************
 * $Log:   S:\3x\src\ntkm\ntdevpp\sampdrv\object.h_v  $
 * 
 *    Rev 1.0   24 Jun 1996 16:24:36   JHUCKINS
 * Initial revision.
 * 
 *    Rev 1.6   03 Jun 1996 07:47:12   JHUCKINS
 * Added Author to file header
 * 
 *    Rev 1.5   31 May 1996 15:50:46   JDELLIOT
 * added robustness features (ASSERT, TRACE, validation of params)
 * cleanup
 * restructure of some interfaces
 *
 */

