////////////////////////////////////////////////////////////////////////////////
// $Header:   S:\3x\src\ntkm\ntdevpp\sampdrv\regkey.cpv   1.0   24 Jun 1996 16:24:28   JHUCKINS  $
// 
// INTEL Corporation Proprietary Information
// 
// This listing is supplied under the terms of a license agreement
// with INTEL Corporation and may not be copied nor disclosed except
// in accordance with the terms of that agreement.
// 
// Copyright (c) 1996 Intel Corporation. All rights reserved.
// 
// $Revision:   1.0  $
// $Date:   24 Jun 1996 16:24:28  $
// $Author:   JHUCKINS  $
// 
// Log at end of file.
// 
// Module Name:	regkey.cpp
// Abstract:	CRegistryKey implementation
// Environment:	Windows NT 3.51/4.0, MS Visual C++ 4.x
// Notes:
// 
////////////////////////////////////////////////////////////////////////////////

#pragma warning( disable: 4201 )	// nonstandard extension used : nameless struct/union
#pragma warning( disable: 4514 )	// unreferenced inline function has been removed
#pragma warning( disable: 4711 )	// function 'function' selected for automatic inline expansion

#include <idrivrpp.h>
#include "debug.h"

////////////////////////////////////////////////////////////////////////////////
// create methods
//
NTSTATUS CRegistryKey::Create(
	IN PCWSTR Name,
	IN PCWSTR RootDirectoryName,
	IN ACCESS_MASK DesiredAccess,
	OUT PULONG Disposition )
{
	// set the name of the root directory object
	if( FALSE == m_RootDirectory.SetName( RootDirectoryName ) )
	{
		TRACE( "CRegistryKey::Create - Failed to set name of root directory file object!" );
		return STATUS_NO_MEMORY;
	}

	// do we have a root directory name
	if( NULL != RootDirectoryName )
	{
		// Create the root directory object
		NTSTATUS Status;
		IO_STATUS_BLOCK IoStatus;
		if( STATUS_SUCCESS != (Status = m_RootDirectory.Create( GENERIC_READ | GENERIC_WRITE, &IoStatus )) )
		{
			TRACE( "CRegistryKey::Create - Failed to create root directory file object!" );
			return Status;
		}
	}

	return Create( Name, DesiredAccess, Disposition );
}

NTSTATUS CRegistryKey::Create(
	IN CUnicodeString& Name,
	IN CUnicodeString& RootDirectoryName,
	IN ACCESS_MASK DesiredAccess,
	OUT PULONG Disposition )
{
	// set the name of the root directory object
	if( FALSE == m_RootDirectory.SetName( RootDirectoryName ) )
	{
		TRACE( "CRegistryKey::Create - Failed to set name of root directory file object!" );
		return STATUS_NO_MEMORY;
	}

	// do we have a root directory name
	if( 0 != RootDirectoryName.GetLength() )
	{
		// Create the root directory object
		NTSTATUS Status;
		IO_STATUS_BLOCK IoStatus;
		if( STATUS_SUCCESS != (Status = m_RootDirectory.Create( GENERIC_READ | GENERIC_WRITE, &IoStatus )) )
		{
			TRACE( "CRegistryKey::Create - Failed to create root directory file object!" );
			return Status;
		}
	}

	return Create( Name, DesiredAccess, Disposition );
}

NTSTATUS CRegistryKey::Create(
	IN CUnicodeString& Name,
	IN ACCESS_MASK DesiredAccess,
	OUT PULONG Disposition )
{
	// set the name of this object
	if( FALSE == SetName( Name ) )
	{
		TRACE( "CRegistryKey::Create - Failed to set name!" );
		return STATUS_NO_MEMORY;
	}

	ASSERT( NULL != Disposition );

	// initialize object attributes
	OBJECT_ATTRIBUTES ObjectAttributes;
	InitializeObjectAttributes(
		&ObjectAttributes,
		m_Name,
		OBJ_CASE_INSENSITIVE | OBJ_OPENIF,
		m_RootDirectory,
		NULL );

	// create/open the key
	return ZwCreateKey(
				&m_Handle,
				DesiredAccess,
				&ObjectAttributes,
				0,		// TitleIndex
				NULL,	// Class
				REG_OPTION_NON_VOLATILE,
				Disposition );
}

NTSTATUS CRegistryKey::Create(
	IN PCWSTR Name,
	IN ACCESS_MASK DesiredAccess,
	OUT PULONG Disposition )
{
	// set the name of this object
	if( FALSE == SetName( Name ) )
	{
		TRACE( "CRegistryKey::Create - Failed to set name!" );
		return STATUS_NO_MEMORY;
	}

	ASSERT( NULL != Disposition );

	// initialize object attributes
	OBJECT_ATTRIBUTES ObjectAttributes;
	InitializeObjectAttributes(
		&ObjectAttributes,
		m_Name,
		OBJ_CASE_INSENSITIVE | OBJ_OPENIF,
		m_RootDirectory,
		NULL );

	// create/open the key
	return ZwCreateKey(
				&m_Handle,
				DesiredAccess,
				&ObjectAttributes,
				0,		// TitleIndex
				NULL,	// Class
				REG_OPTION_NON_VOLATILE,
				Disposition );
}

NTSTATUS CRegistryKey::Create( 
	IN CUnicodeString& Name,
	OUT PULONG Disposition )
{
	return Create( Name, KEY_ALL_ACCESS, Disposition );
}

NTSTATUS CRegistryKey::Create( 
	IN PCWSTR Name,
	OUT PULONG Disposition )
{
	return Create( Name, KEY_ALL_ACCESS, Disposition );
}

////////////////////////////////////////////////////////////////////////////////
// open method
//
NTSTATUS CRegistryKey::Open( IN ACCESS_MASK DesiredAccess /* = KEY_ALL_ACCESS */ )
{
	OBJECT_ATTRIBUTES ObjectAttributes;
	InitializeObjectAttributes(
		&ObjectAttributes,
		m_Name,
		OBJ_CASE_INSENSITIVE,
		m_RootDirectory,
		NULL );

	return ZwOpenKey(
			&m_Handle,
			DesiredAccess,
			&ObjectAttributes );
}

////////////////////////////////////////////////////////////////////////////////
// query methods
//
NTSTATUS CRegistryKey::QueryValueULONG(
	IN PUNICODE_STRING ValueName,
	OUT PULONG Value )
{
	// First see how big of a buffer we need
	PKEY_VALUE_PARTIAL_INFORMATION Information = NULL;
	ULONG ResultLength = 0;
	NTSTATUS Status = QueryValue(
						ValueName,
						KeyValuePartialInformation,
						&Information,
						0,
						&ResultLength );

	// we expect to fail, but did we fail for the wrong reason
	if( STATUS_BUFFER_TOO_SMALL != Status )
	{
		ASSERT( ! NT_SUCCESS( Status ) );
		TRACE( "CRegistryKey::QueryValueULONG - Failed to retrieve buffer size!" );
		return Status;
	}

	// ResultLength has the necessary size of the buffer.
	// allocate the buffer
	Information = PKEY_VALUE_PARTIAL_INFORMATION( new char[ResultLength] );
	if( NULL == Information )
	{
		TRACE( "CRegistryKey::QueryValueULONG - Failed to allocate Information!" );
		return STATUS_NO_MEMORY;
	}

	// now get the data for real
	Status = QueryValue(
				ValueName,
				KeyValuePartialInformation,
				Information,
				ResultLength,
				&ResultLength );

	// assign the value
	if( STATUS_SUCCESS == Status )
	{
		// validate the value type
		if( REG_DWORD == Information->Type )
		{
			*Value = *PULONG( PCHAR( Information->Data ) );
		}
		else
		{
			TRACE( "RegistryKey::QueryValueULONG - Failed because an invalid value type was returned!" );
			Status = STATUS_INVALID_PARAMETER;
		}
	}

	// Don't forget to free the buffer
	delete Information;

	return Status;
}

NTSTATUS CRegistryKey::QueryValueString(
	IN PUNICODE_STRING ValueName,
	OUT CUnicodeString& Value )
{
	// First see how big of a buffer we need	
	PKEY_VALUE_PARTIAL_INFORMATION Information = NULL;
	ULONG ResultLength = 0;
	NTSTATUS Status = QueryValue(
						ValueName,
						KeyValuePartialInformation,
						&Information,
						0,
						&ResultLength );

	// we expect to fail, but did we fail for the wrong reason
	if( STATUS_BUFFER_TOO_SMALL != Status )
	{
		ASSERT( ! NT_SUCCESS( Status ) );
		TRACE( "CRegistryKey::QueryValueString - Failed to retrieve buffer size!" );
		return Status;
	}

	// ResultLength has the necessary size of the buffer.
	// allocate the buffer
	Information = PKEY_VALUE_PARTIAL_INFORMATION( new char[ ResultLength ] );
	if( NULL == Information )
	{
		TRACE( "CRegistryKey::QueryValueString - Failed to allocate Information!" );
		return STATUS_NO_MEMORY;
	}

	// now get the data for real
	Status = QueryValue(
				ValueName,
				KeyValuePartialInformation,
				Information,
				ResultLength,
				&ResultLength );

	// assign the value
	if( STATUS_SUCCESS == Status )
	{
		if( REG_SZ == Information->Type )
		{
			// initialize the CUnicodeString
			// @@@ do we always want this to be in the paged pool
			if( FALSE == Value.Copy( PWSTR( Information->Data ), PagedPool ) )
			{
				TRACE( "CRegistryKey::QueryValueString - Failed to Copy new value to String! ");
				Status = STATUS_NO_MEMORY;
			}
		}
		else
		{
			TRACE( "RegistryKey::QueryValueString - Failed because an invalid value type was returned!" );
			Status = STATUS_INVALID_PARAMETER;
		}
	}

	// don't forget to free the buffer
	delete Information;

	return Status;
}

BOOLEAN CRegistryKey::IsValid( void )
{
	return BOOLEAN( m_RootDirectory.IsValid() && CNamedObject::IsValid() );
}

/////////////////////////////////////////////////////////////////////////////
// $Log:   S:\3x\src\ntkm\ntdevpp\sampdrv\regkey.cpv  $
// 
//    Rev 1.0   24 Jun 1996 16:24:28   JHUCKINS
// Initial revision.
// 
//    Rev 1.7   18 Jun 1996 15:11:40   JHUCKINS
// Fix QueryValueULONG.  It was failing size validation for ResultLength
// in error, so didn't get the value
// 
//    Rev 1.6   03 Jun 1996 07:47:12   JHUCKINS
// Added Author to file header
// 
//    Rev 1.5   02 Jun 1996 15:59:28   JHUCKINS
// Added Create Members
// 
//    Rev 1.4   31 May 1996 16:13:34   JHUCKINS
// Removed default constructor
// 
//    Rev 1.3   31 May 1996 15:50:50   JDELLIOT
// added robustness features (ASSERT, TRACE, validation of params)
// cleanup
// restructure of some interfaces
// 
