//	******************************************************
//	*
//	*  INTEL Corporation Proprietary Information
//	*
//	*  This listing is supplied under the terms of a
//	*  license agreement  with INTEL Corporation and
//	*  may not be copied nor disclosed except
//	*  in accordance with the terms of that agreement.
//	*
//	*  Copyright (c) 1996 Intel Corporation. All rights
//	*  reserved.
//	*
//	*  Module Name:	SAMPDEV.CPP
//	*  Abstract:	SampleDevice Device object class implementation
//	*  Environment:
//	*
//	*  Notes:
//	* <Assumptions>
//	* <Other of interest>
//	*
//	******************************************************
#pragma warning (disable : 4201 )	// nonstandard extension used : nameless struct/union 
#pragma warning (disable : 4514 )	// unreferenced inline function has been removed

#include "sampdev.h"
#include "debug.h"

///////////////////////////////////////////////////////////////////////////////
// ctor/dtor
///////////////////////////////////////////////////////////////////////////////
CSampleDevice::CSampleDevice()
	:	CPciDevice( NULL ),
		m_InputDmaSize( 0 ),
		m_OutputDmaSize( 0 )
{
}

CSampleDevice::CSampleDevice( IN CDriverObject *DriverObject )
	:	CPciDevice( DriverObject ),
		m_InputDmaSize( 256 ),
		m_OutputDmaSize( 256 )
{
}

CSampleDevice::~CSampleDevice()
{
}

NTSTATUS CSampleDevice::OnInitialize( IN PUNICODE_STRING RegistryPath )
{
	NTSTATUS	status = CPciDevice::OnInitialize( RegistryPath );

	if( status != STATUS_SUCCESS )
		return status;

	// Initialize the adapter objects
	// Each gets a page-size, cache-enabled common buffer
	DEVICE_DESCRIPTION	DeviceDescription;
	
	GetDeviceDescription( &DeviceDescription );
		
	TRACE("OnInitialize");

	if( STATUS_SUCCESS != 
		(status = m_ReadAdapter.Initialize( 
						RegistryPath,			// RegistryPath
						m_InputDmaSize,			// Size of common buffer
						0x1000,					// Alignment
						FALSE,					// Cache enabled common buffer
						&DeviceDescription )) )	// Device description
	{
		TRACE("OnInitialize Failed to initialize read adapter!");
		return status;
	}
	ASSERT( 0L == m_ReadAdapter.GetCommonBufferLogicalAddress().HighPart );

	if( STATUS_SUCCESS != 
		(status = m_WriteAdapter.Initialize( 
						RegistryPath,			// RegistryPath
						m_OutputDmaSize,		// Size of common buffer
						0x1000,					// Alignment
						FALSE,					// Cache enabled common buffer
						&DeviceDescription )) )	// Device description
	{
		TRACE("OnInitialize Failed to initialize write adapter!");
		return status;
	}
	ASSERT( 0L == m_WriteAdapter.GetCommonBufferLogicalAddress().HighPart );
	TRACE("Read Common Buffer Phys = %X, Virt = %X", 
			m_ReadAdapter.GetCommonBufferLogicalAddress().LowPart,
			m_ReadAdapter.GetCommonBuffer() );
	TRACE("Write Common Buffer Phys = %X, Virt = %X", 
			m_WriteAdapter.GetCommonBufferLogicalAddress().LowPart,
			m_WriteAdapter.GetCommonBuffer() );

	// Setup the custom DPCs
	m_DpcForWriteIsr.InitializeDpcRequest( DpcForWriteIsr, this );
	m_DpcForReadIsr.InitializeDpcRequest( DpcForReadIsr, this );

	return STATUS_SUCCESS;
}

NTSTATUS CSampleDevice::OnIrpMjDeviceControl( IN PIRP Irp )
{
	TRACE("OnIrpMjDeviceControl");
	PIO_STACK_LOCATION request = IoGetCurrentIrpStackLocation( Irp );
	NTSTATUS Status = STATUS_INVALID_PARAMETER;
	ULONG Information = 0;

	CompleteRequest( Irp, Status, Information ); 
	return Status;
}

NTSTATUS CSampleDevice::OnIrpMjRead( IN PIRP Irp )
{
	TRACE("OnIrpMjRead");

	NTSTATUS Status = STATUS_PENDING;

	// If we're not busy, start the IRP
	if( !m_ReadQueue.InsertData( Irp ) )
	{
		// Only one outstanding write request supported
		if( NULL != m_ReadRequest.Irp )
		{
			TRACE("OnIrpMjRead - Invalid Device Request!!!");
			Status = STATUS_INVALID_DEVICE_REQUEST;
		}
		else
		{
			PIO_STACK_LOCATION request = IoGetCurrentIrpStackLocation( Irp );

			// Cache the Irp
			m_ReadRequest.SpinLock.Acquire();
			m_ReadRequest.Irp = Irp;
			m_ReadRequest.Buffer = MmGetSystemAddressForMdl( Irp->MdlAddress );
			m_ReadRequest.Length = request->Parameters.Read.Length;
			m_ReadRequest.SpinLock.Release();

			// Synchronize device manipulation for reading from device
			m_Interrupt.SynchronizeExecutionForRead();
		}
	}

	if( STATUS_PENDING != Status )
		CompleteRequest( Irp, Status );
	else
		IoMarkIrpPending( Irp );

	return Status;
}

NTSTATUS CSampleDevice::OnIrpMjWrite( IN PIRP Irp )
{
	TRACE("OnIrpMjWrite");

	NTSTATUS Status = STATUS_PENDING;

	// If we're not busy, start the IRP
	if( !m_WriteQueue.InsertData( Irp ) )
	{
		// Only one outstanding write request supported
		if( NULL != m_WriteRequest.Irp )
		{
			TRACE("OnIrpMjWrite - Invalid Device Request!!!");
			Status = STATUS_INVALID_DEVICE_REQUEST;
		}
		else
		{
			PIO_STACK_LOCATION request = IoGetCurrentIrpStackLocation( Irp );

			// Cache the Irp
			m_WriteRequest.SpinLock.Acquire();
			m_WriteRequest.Irp = Irp;
			m_WriteRequest.Buffer = MmGetSystemAddressForMdl( Irp->MdlAddress );
			m_WriteRequest.Length = request->Parameters.Write.Length;
			m_WriteRequest.SpinLock.Release();

			// Synchronize device manipulation for writing to device
			m_Interrupt.SynchronizeExecutionForWrite();
		}
	}

	if( STATUS_PENDING != Status )
		CompleteRequest( Irp, Status );
	else
		IoMarkIrpPending( Irp );

	return Status;
}

VOID CSampleDevice::OnUnload()
{
	TRACE("OnUnload");

	// Don't EVER forget to call CAdapterObject::Unload()
	m_ReadAdapter.Unload();
	m_WriteAdapter.Unload();
	DisconnectInterrupt();
	CPciDevice::OnUnload();
}

BOOLEAN CSampleDevice::OnInterrupt()
{
	TRACE("OnInterrupt");
	NTSTATUS Status = STATUS_SUCCESS;

//	if( !OurInterrupt )
//		return FALSE;

	// Has a write happened?		
//	if( IsWriteInterrupt )
//	{	
		// Queue DPC for write ISR
		if( !m_DpcForWriteIsr.InsertQueue( (PVOID)Status ) )
		{
			TRACE( "OnInterrupt - Failed to queue Write DPC!" );
		}
//	}

	// Has a read happened?		
//	if( IsReadInterrupt )
//	{	
		// Queue DPC for read ISR
		if( !m_DpcForReadIsr.InsertQueue( (PVOID)Status ) )
		{
			TRACE( "OnInterrupt - Failed to queue Read DPC!" );
		}
//	}

	TRACE("Leaving OnInterrupt");
	return TRUE;
}

VOID CSampleDevice::GetDeviceDescription( OUT PDEVICE_DESCRIPTION DeviceDescription )
{
    DeviceDescription->Version				= DEVICE_DESCRIPTION_VERSION;
    DeviceDescription->Master				= TRUE,				// Busmaster Device
    DeviceDescription->ScatterGather		= FALSE,			// No scatter/gather
    DeviceDescription->DemandMode			= FALSE,			// Don't use system DMA controller
    DeviceDescription->AutoInitialize		= FALSE,			// Don't use system DMA controller
    DeviceDescription->Dma32BitAddresses	= TRUE,				// 32 bit addresses
    DeviceDescription->IgnoreCount			= FALSE,			// IgnoreCount
    DeviceDescription->Reserved1			= FALSE,			// Reserved1
    DeviceDescription->Reserved2			= FALSE,			// Reserved2
    DeviceDescription->BusNumber			= m_ulBusNumber,	// BusNumber
    DeviceDescription->DmaChannel			= 0,				// No system DMA channel involved
    DeviceDescription->InterfaceType		= PCIBus,			// BusType
    DeviceDescription->DmaWidth				= Width32Bits,		// 32-bit wide Dma
    DeviceDescription->DmaSpeed				= MaximumDmaSpeed,
    DeviceDescription->MaximumLength		= 8196,				// MaximumLength
    DeviceDescription->DmaPort				= 0;				// DmaPort
}

BOOLEAN CSampleDevice::OnSynchronizeExecutionForRead()
{
	// This is where you setup the device for a read from the device
	return TRUE;
}

BOOLEAN CSampleDevice::OnSynchronizeExecutionForWrite()
{
	// This is where you setup the device for a write to the device
	return TRUE;
}

VOID CSampleDevice::OnDpcForWriteIsr( IN NTSTATUS Status )
{
	TRACE("Entering OnDpcForWriteIsr");
	ASSERT( NULL != m_WriteRequest.Irp );

	// Get the next IRP in the queue
	PIRP NextIrp = PIRP(m_WriteQueue.RemoveData());
	PIRP CurrentIrp;
	ULONG Information;

	// Update the IO Request
	m_WriteRequest.SpinLock.AcquireAtDpcLevel();
	CurrentIrp = m_WriteRequest.Irp;
	Information = STATUS_SUCCESS == Status ? m_WriteRequest.Length : 0;
	m_WriteRequest.Irp = NextIrp;
	m_WriteRequest.SpinLock.ReleaseFromDpcLevel();

	// Complete the current WriteIrp
	CompleteRequest( 
		CurrentIrp, 
		Status, 
		Information );

	if( NextIrp )
	{
		PIO_STACK_LOCATION request = IoGetCurrentIrpStackLocation( NextIrp );
		m_WriteRequest.Buffer = MmGetSystemAddressForMdl( NextIrp->MdlAddress );
		m_WriteRequest.Length = request->Parameters.Write.Length;

		// This is where you'd perform the setup for the next write to the device
	}

	TRACE("Leaving OnDpcForWriteIsr");
}

VOID CSampleDevice::OnDpcForReadIsr( IN NTSTATUS Status )
{
	TRACE("Entering OnDpcForReadIsr");
	ASSERT( NULL != m_ReadRequest.Irp );
	ASSERT( NULL != m_ReadRequest.Buffer );

	if( STATUS_SUCCESS == Status )
	{
		// Copy the data from the device read buffer to the user
		// buffer.
		RtlMoveMemory( m_ReadRequest.Buffer, m_ReadAdapter.GetCommonBuffer(), m_ReadRequest.Length );

	}

	// Get the next IRP in the queue
	PIRP NextIrp = PIRP(m_ReadQueue.RemoveData());
	PIRP CurrentIrp;
	ULONG Information;

	// Update the IO Request
	m_ReadRequest.SpinLock.AcquireAtDpcLevel();
	CurrentIrp = m_ReadRequest.Irp;
	Information = STATUS_SUCCESS == Status ? m_ReadRequest.Length : 0;
	m_ReadRequest.Irp = NextIrp;
	m_ReadRequest.SpinLock.ReleaseFromDpcLevel();

	// Complete the request
	CompleteRequest( 
		CurrentIrp,
		Status, 
		Information );

	if( NextIrp )
	{
		PIO_STACK_LOCATION request = IoGetCurrentIrpStackLocation( NextIrp );
		m_ReadRequest.Buffer = MmGetSystemAddressForMdl( NextIrp->MdlAddress );
		m_ReadRequest.Length = request->Parameters.Read.Length;

		// This is where you'd perform the setup for the next read from the device
	}

	TRACE("Leaving OnDpcForReadIsr");
}

BOOLEAN CSampleDevice::OnErrorInterrupt()
{
	TRACE( "OnErrorInterrupt Entered" );

	// Queue up DPCs to complete the outstanding Irps
	if( NULL != m_WriteRequest.Irp )
		m_DpcForWriteIsr.InsertQueue( (PVOID)STATUS_DEVICE_DATA_ERROR );
	if( NULL != m_ReadRequest.Irp )
		m_DpcForReadIsr.InsertQueue( (PVOID)STATUS_DEVICE_DATA_ERROR );

	return TRUE;
}

///////////////////////////////////////
//	Class statics
///////////////////////////////////////
/* static */
VOID CSampleDevice::DpcForWriteIsr(
	IN PKDPC /* Dpc */,
	IN PVOID DeferredContext,
	IN PVOID Status,
	IN PVOID /* SystemArgument2 */ )
{
	((CSampleDevice*)DeferredContext)->OnDpcForWriteIsr( (NTSTATUS)Status );
}

/* static */
VOID CSampleDevice::DpcForReadIsr(
	IN PKDPC /* Dpc */,
	IN PVOID DeferredContext,
	IN PVOID Status,
	IN PVOID /* SystemArgument2 */)
{
	((CSampleDevice*)DeferredContext)->OnDpcForReadIsr( (NTSTATUS)Status );
}

///////////////////////////////////////
//	Utility functions
///////////////////////////////////////
VOID CompleteRequest( 
			IN PIRP Irp,
			IN NTSTATUS Status,
			IN ULONG Information /* = 0 */,
			IN CCHAR PriorityBoost /* = IO_NO_INCREMENT */ )
{
	TRACE( "CompletRequest Entered" );
	Irp->IoStatus.Status = Status;
	Irp->IoStatus.Information = Information;
	IoCompleteRequest( Irp, PriorityBoost );
}
