//	******************************************************
//	*  Module Name:	SAMPDEV.H
//	*  Abstract:	CSampleDevice Device object class declaration
//	*  Environment:
//	* 
//	*  Notes:
//	* <Assumptions>
//	* <Other of interest>
//	*
//	******************************************************
#ifndef SAMPDEV_H
#define SAMPDEV_H

#include <idrivrpp.h>
#include "pcidev.h"

//	****************************
//	*	Predeclarations
//	****************************

//	****************************
//	*	Types
//	****************************
struct CIoRequest
{
	// ctor/dtor	
	CIoRequest();

	PIRP		Irp;
	PVOID		Buffer;
	ULONG		Length;
	CSpinLock	SpinLock;
};
//*****************************************************************
//*	class CIoRequest Inlines
//*****************************************************************
inline CIoRequest::CIoRequest()
	:	Irp( NULL ),
		Buffer( NULL ),
		Length( Length )
{
}

#define PIO_REQUEST CIoRequest*

//	****************************
//	*	Macros
//	****************************
#define min( a, b ) (ULONG)a < (ULONG)b ? a : b

//	****************************
//	*	Class: CSampleDevice
//	****************************
class CSampleDevice : public CPciDevice
{
	public:
		CSampleDevice( IN CDriverObject *DriverObject );

		virtual ~CSampleDevice();

	protected:
		CSampleDevice();
		CSampleDevice( IN const CSampleDevice &right );

		//	Virtual from CDeviceObject
		virtual NTSTATUS OnInitialize( IN PUNICODE_STRING RegistryPath );

		//	Virtual from CDeviceObject
		virtual NTSTATUS OnIrpMjDeviceControl( IN PIRP Irp );
		virtual NTSTATUS OnIrpMjRead( IN PIRP Irp );
		virtual NTSTATUS OnIrpMjWrite( IN PIRP Irp );
		virtual VOID OnUnload(VOID);
		virtual BOOLEAN OnInterrupt(VOID);
		virtual VOID GetDeviceDescription( OUT PDEVICE_DESCRIPTION DeviceDescription );

		// called by CInterruptObject::SynchronizeExecutionForXXX() to
		// synchronize device I/O programming.
		virtual BOOLEAN OnSynchronizeExecutionForRead( VOID );
		virtual BOOLEAN OnSynchronizeExecutionForWrite( VOID );

	#ifdef NOTDEF
		virtual NTSTATUS OnIrpMjCreate( IN PIRP Irp );
		virtual NTSTATUS OnIrpMjClose( IN PIRP Irp );
		virtual NTSTATUS OnIrpMjCleanup( IN PIRP Irp );
		virtual NTSTATUS OnIrpMjFlushBuffers( IN PIRP Irp );
		virtual NTSTATUS OnIrpMjInternalDeviceControl( IN PIRP Irp );
		virtual NTSTATUS OnIrpMjQueryInformation( IN PIRP Irp );
		virtual NTSTATUS OnIrpMjSetInformation( IN PIRP Irp );
		virtual VOID OnCancel( IN PIRP Irp );
	#endif

		// The Dpc handlers
		VOID OnDpcForWriteIsr( IN NTSTATUS Status );
		VOID OnDpcForReadIsr( IN NTSTATUS Status );

		// The error interrupt handler
		BOOLEAN OnErrorInterrupt(VOID);

		////////////////////////////
		// The custom DPC routines
		////////////////////////////
		static VOID DpcForWriteIsr(
						IN PKDPC Dpc,
						IN PVOID DeferredContext,
						IN PVOID Status,
						IN PVOID SystemArgument2 );

		static VOID DpcForReadIsr(
						IN PKDPC Dpc,
						IN PVOID DeferredContext,
						IN PVOID Status,
						IN PVOID SystemArgument2 );

	private:
  		CIoRequest			m_ReadRequest;
		CIoRequest			m_WriteRequest;
		CAdapterObject		m_ReadAdapter;
		CAdapterObject		m_WriteAdapter;
		CDeviceQueue		m_ReadQueue;
		CDeviceQueue		m_WriteQueue;
		ULONG				m_InputDmaSize;
		ULONG				m_OutputDmaSize;

		// The custom DPCs for setting up for device reads and writes
		CCustomDpcObject	m_DpcForWriteIsr;
		CCustomDpcObject	m_DpcForReadIsr;
};

//*****************************************************************
//*	C routine prototypes
//*****************************************************************
VOID CompleteRequest( 
			IN PIRP Irp,
			IN NTSTATUS Status,
			IN ULONG Information = 0,
			IN CCHAR PriorityBoost = IO_NO_INCREMENT);

#endif
