////////////////////////////////////////////////////////////////////////////////
// $Header:   S:\3x\src\ntkm\ntdevpp\sampdrv\spinlock.h_v   1.0   24 Jun 1996 16:24:34   JHUCKINS  $
// 
// INTEL Corporation Proprietary Information
// 
// This listing is supplied under the terms of a license agreement
// with INTEL Corporation and may not be copied nor disclosed except
// in accordance with the terms of that agreement.
// 
// Copyright (c) 1996 Intel Corporation. All rights reserved.
// 
// $Revision:   1.0  $
// $Date:   24 Jun 1996 16:24:34  $
// $Author:   JHUCKINS  $
// 
// Log at end of file.
// 
// Module Name:		spinlock.h
// Abstract:		CSpinLock Inteface
// Environment:		Windows NT 3.51/4.0, MS Visual C++ 2.x
// Notes:
// 
////////////////////////////////////////////////////////////////////////////////

#ifndef spinlock_h
#define spinlock_h

#include <idrivrpp.h>

class CSpinLock
{
public:
	// ctor/dtor
	CSpinLock( void );
	virtual ~CSpinLock( void );

	// operators
	operator KSPIN_LOCK( void );
	operator PKSPIN_LOCK( void );

	// methods
	VOID Acquire( void );
	VOID AcquireAtDpcLevel( void );
	VOID Release( void );
	VOID ReleaseFromDpcLevel( void );

private:
	KSPIN_LOCK	m_SpinLock;
	KIRQL		m_OldIrql;
	BOOLEAN		m_Acquired;
};

////////////////////////////////////////////////////////////////////////////////
// inline functions
//
inline CSpinLock::CSpinLock( void )
	:	m_Acquired( FALSE )
{
	KeInitializeSpinLock( &m_SpinLock );
	return;
}

inline CSpinLock::~CSpinLock( void )
{
	if( TRUE == m_Acquired )
	{
		KIRQL Irql = KeGetCurrentIrql();
		if( DISPATCH_LEVEL == Irql )
		{
			ReleaseFromDpcLevel();
		}
		else if( DISPATCH_LEVEL > Irql )
		{
			Release();
		}
		else
		{
			TRACE( "CSpinLock::~CSpinLock - spin lock object deleted without being release!" );
		}
	}

	return;
}

inline CSpinLock::operator KSPIN_LOCK( void )
{
	return m_SpinLock;
}

inline CSpinLock::operator PKSPIN_LOCK( void )
{
	return &m_SpinLock;
}

inline VOID CSpinLock::Acquire( void )
{
	ASSERT_IRQL_LE( DISPATCH_LEVEL );
	KeAcquireSpinLock( &m_SpinLock, &m_OldIrql );
	m_Acquired = TRUE;
	return;
}

inline VOID CSpinLock::AcquireAtDpcLevel( void )
{
	ASSERT_IRQL_EQ( DISPATCH_LEVEL );
	KeAcquireSpinLockAtDpcLevel( &m_SpinLock );
	m_Acquired = TRUE;
	return;
}

inline VOID CSpinLock::Release( void )
{
	ASSERT_IRQL_LE( DISPATCH_LEVEL );
	KeReleaseSpinLock( &m_SpinLock, m_OldIrql );
	m_Acquired = FALSE;
	return;
}

inline VOID CSpinLock::ReleaseFromDpcLevel( void )
{
	ASSERT_IRQL_EQ( DISPATCH_LEVEL );
	KeReleaseSpinLockFromDpcLevel( &m_SpinLock );
	m_Acquired = FALSE;
	return;
}

#endif spinlock_h

/******************************************************************************
 * $Log:   S:\3x\src\ntkm\ntdevpp\sampdrv\spinlock.h_v  $
 * 
 *    Rev 1.0   24 Jun 1996 16:24:34   JHUCKINS
 * Initial revision.
 * 
 *    Rev 1.2   03 Jun 1996 07:47:12   JHUCKINS
 * Added Author to file header
 * 
 *    Rev 1.1   31 May 1996 15:50:42   JDELLIOT
 * added robustness features (ASSERT, TRACE, validation of params)
 * cleanup
 * restructure of some interfaces
 * 
 *    Rev 1.0   08 May 1996 15:29:10   JDELLIOT
 * Initial revision.
 * 
 */
