////////////////////////////////////////////////////////////////////////////////
// $Header:   S:\3x\src\ntkm\ntdevpp\sampdrv\thread.h_v   1.0   24 Jun 1996 16:24:34   JHUCKINS  $
// 
// INTEL Corporation Proprietary Information
// 
// This listing is supplied under the terms of a license agreement
// with INTEL Corporation and may not be copied nor disclosed except
// in accordance with the terms of that agreement.
// 
// Copyright (c) 1996 Intel Corporation. All rights reserved.
// 
// $Revision:   1.0  $
// $Date:   24 Jun 1996 16:24:34  $
// $Author:   JHUCKINS  $
// 
// Log at end of file.
// 
// Module Name:	thread.h
// Author:		Jeff Huckins
// Abstract:	CThread Inteface
// Environment:	Windows NT 3.51/4.0, MS Visual C++ 2.x
// Notes:
// 
////////////////////////////////////////////////////////////////////////////////

#ifndef thread_h
#define thread_h

class CThread : public CObject
{
public:
	// ctor/dtor
	CThread();

	// creates the thread
	NTSTATUS Create(
		IN ULONG DesiredAccess = 0L,
		IN PVOID StartContext = NULL,
		IN PVOID TerminateContext = NULL );

	// terminates the thread
	static NTSTATUS Terminate( NTSTATUS ExitStatus );

	// adjust the thread priority
	LONG SetBasePriority( IN LONG Increment );
	LONG SetPriority( IN LONG Increment );

	// get the objects pointer (adds a reference to the object)
	virtual NTSTATUS GetObjectPtr( IN PVOID& Object );

	// get the status of the terminated thread
	NTSTATUS GetExitStatus( VOID );

	// cast operator
	operator PKTHREAD();

protected:
	virtual NTSTATUS OnStart( IN PVOID StartContext ) = 0;
	virtual VOID OnTerminate( IN PVOID TerminationContext );

private:
	static VOID StartRoutine( IN CThread* This );

	// members
	PVOID		m_StartContext;
	PVOID		m_TerminationContext;
	NTSTATUS	m_ExitStatus;
};

////////////////////////////////////////////////////////////////////////////////
// inlines
//
inline CThread::CThread()
	:	m_StartContext( NULL ),
		m_TerminationContext( NULL ),
		m_ExitStatus( STATUS_SUCCESS )
{
	return;
}

inline NTSTATUS CThread::Create(
	IN ULONG DesiredAccess /* = 0L */,
	IN PVOID StartContext /* = NULL */,
	IN PVOID TerminationContext /* = NULL */ )
{
	ASSERT_IRQL_EQ( PASSIVE_LEVEL );

	// store contexts
	m_StartContext = StartContext;
	m_TerminationContext = TerminationContext;

	// create the thread
	return PsCreateSystemThread(
		&m_Handle,
		DesiredAccess,
		NULL,
		NULL,
		NULL,
		PKSTART_ROUTINE( StartRoutine ),
		this );
}

inline NTSTATUS CThread::Terminate( NTSTATUS ExitStatus )
{
	ASSERT_IRQL_EQ( PASSIVE_LEVEL );
	return PsTerminateSystemThread( ExitStatus );
}

inline LONG CThread::SetBasePriority( IN LONG Increment )
{
	ASSERT_IRQL_EQ( PASSIVE_LEVEL );
	return KeSetBasePriorityThread( PKTHREAD( this ), Increment );
}

inline LONG CThread::SetPriority( IN LONG Increment )
{
	ASSERT_IRQL_EQ( PASSIVE_LEVEL );
	return KeSetPriorityThread( PKTHREAD( this ), Increment );
}

inline NTSTATUS CThread::GetExitStatus( VOID )
{
	return m_ExitStatus;
}

inline CThread::operator PKTHREAD()
{
	// get the object pointer
	PVOID Object;
	GetObjectPtr( Object );

	return PKTHREAD( Object );
}

#endif thread_h

/*******************************************************************************
 * $Log:   S:\3x\src\ntkm\ntdevpp\sampdrv\thread.h_v  $
 * 
 *    Rev 1.0   24 Jun 1996 16:24:34   JHUCKINS
 * Initial revision.
 * 
 *    Rev 1.8   03 Jun 1996 07:47:10   JHUCKINS
 * Added Author to file header
 * 
 *    Rev 1.7   31 May 1996 15:50:32   JDELLIOT
 * added robustness features (ASSERT, TRACE, validation of params)
 * cleanup
 * restructure of some interfaces
 * 
 *    Rev 1.6   13 May 1996 10:04:46   JHUCKINS
 * c52 bootload complete
 * 
 *    Rev 1.5   10 May 1996 23:10:00   JHUCKINS
 * implement CObject to derive CThread from
 * 
 *    Rev 1.4   02 May 1996 16:23:40   JHUCKINS
 * Added TERMINATION_ROUTINE callback that is registered by owner object.
 * 
 *    Rev 1.3   02 May 1996 15:58:20   JHUCKINS
 * No more m_bDynamic
 * 
 *    Rev 1.2   01 May 1996 09:01:10   JHUCKINS
 * Added virtual OnTerminate() method and m_bDynamic member so that the
 * virtual OnTerminate() method can delete this object if it was dynamically
 * allocated, as indicated by the constructor argument.
 * 
 *    Rev 1.1   30 Apr 1996 09:36:50   JHUCKINS
 * Added defaults to arguments for Create()
 * 
 *    Rev 1.0   30 Apr 1996 08:54:40   JHUCKINS
 * Initial revision.
 */
