////////////////////////////////////////////////////////////////////////////////
// $Header:   S:\3x\src\ntkm\ntdevpp\sampdrv\uldevice.h_v   1.0   24 Jun 1996 16:24:32   JHUCKINS  $
// 
// INTEL Corporation Proprietary Information
// 
// This listing is supplied under the terms of a license agreement
// with INTEL Corporation and may not be copied nor disclosed except
// in accordance with the terms of that agreement.
// 
// Copyright (c) 1996 Intel Corporation. All rights reserved.
// 
// $Revision:   1.0  $
// $Date:   24 Jun 1996 16:24:32  $
// $Author:   JHUCKINS  $
// 
// Log at end of file.
// 
// Module Name:	uldevice.h
// Author:		Jeff Huckins
// Abstract:	CUpperDeviceObject Interface
// Environment:	Windows NT 3.51/4.0, MS Visual C++ 2.x
// Notes:
//		CDeviceObject which attaches to a lower-level device. The destructor
//		must call IoDetachDevice() in order to decrement the reference count
//		for the target device object.
// 
////////////////////////////////////////////////////////////////////////////////

#ifndef uldevice_h
#define uldevice_h

#include "device.h"

class CUpperDeviceObject : public CDeviceObject 
{
public:
	CUpperDeviceObject( IN CDriverObject* DriverObject );
	virtual ~CUpperDeviceObject();

protected:
	// attachDevice attaches the caller's device object to a named target
	// device object, so that I/O requests bound for the target device are
	// routed first to the caller.
	NTSTATUS AttachDeviceObject( IN PUNICODE_STRING TargetDeviceName );
	NTSTATUS AttachDeviceObject( IN PDEVICE_OBJECT TargetDeviceObject );
	NTSTATUS GetDeviceObjectPointer(
				IN PUNICODE_STRING TargetDeviceName,
				IN ACCESS_MASK DesiredAccess = FILE_ALL_ACCESS,
				BOOLEAN AdjustStackSize = TRUE );

	// detachDeviceObject releases an attachment between the caller's device
	// object and a lower driver's device object.
	VOID DetachDeviceObject(VOID);

	PFILE_OBJECT GetFileObject(VOID);

	// calls IoBuildDeviceIoControlRequest() passing m_AttachedDevice as
	// target DEVICE_OBJECT.
	PIRP BuildDeviceIoControlRequest(
      	IN ULONG IoControlCode, 
      	IN PVOID InputBuffer, 
      	IN ULONG InputBufferLength, 
      	IN PVOID OutputBuffer, 
      	IN ULONG OutputBufferLength, 
      	IN BOOLEAN bInternalDeviceIoControl, 
      	IN PKEVENT Event, 
      	OUT PIO_STATUS_BLOCK StatusBlock );

	// IoCallDriver returns the NTSTATUS value that a lower driver set in the
	// IO status block for the given request or STATUS_PENDING if the request
	// was queued for additional processing.
	// Comments:
	//		IoCallDriver assigns the DeviceObject input parameter to the device
	//		object field of the IRP stack location for the next lower driver.
	//		An IRP passed in a call to IoCallDriver becomes inaccessible to the
	//		higher-level driver, unless the higher-level driver has set up its
	//		IoCompletion routine for the IRP with IoSetCompletionRoutine. If it
	//		does, the IRP input to the driver-supplied IoCompletion routine has
	//		its I/O status block set by the lower driver(s) and all lower-level
	//		driver(s)' I/O stack locations filled with zeros.
	NTSTATUS CallDriver( IN PIRP Irp );

	// calls IoSetHardErrorOrVerifyDevice().
	// Passes m_AttachedDeviceObject as target.
	VOID SetHardErrorOrVerifyDevice( IN PIRP Irp );

	CCHAR GetAttachedDeviceStackSize( VOID );

private:
	// lower level driver device object that is obtained in call to Attach().
	PDEVICE_OBJECT	m_AttachedDeviceObject;
	PFILE_OBJECT	m_FileObject;
};

////////////////////////////////////////////////////////////////////////////////
// inlines
//
inline
NTSTATUS CUpperDeviceObject::CallDriver( IN PIRP Irp )
{
	ASSERT_IRQL_LE( DISPATCH_LEVEL );
	ASSERT( NULL != m_AttachedDeviceObject );
	return IoCallDriver( m_AttachedDeviceObject, Irp );
}

inline
VOID CUpperDeviceObject::SetHardErrorOrVerifyDevice( IN PIRP Irp )
{
	ASSERT_IRQL_LE( DISPATCH_LEVEL );
	IoSetHardErrorOrVerifyDevice( Irp, m_AttachedDeviceObject );
	return;
}

inline
CCHAR CUpperDeviceObject::GetAttachedDeviceStackSize( void )
{
	return (NULL != m_AttachedDeviceObject) ?
		m_AttachedDeviceObject->StackSize : 0;
}

inline
PFILE_OBJECT CUpperDeviceObject::GetFileObject()
{
	return m_FileObject;
}

#endif uldevice_h

/*******************************************************************************
 * $Log:   S:\3x\src\ntkm\ntdevpp\sampdrv\uldevice.h_v  $
 * 
 *    Rev 1.0   24 Jun 1996 16:24:32   JHUCKINS
 * Initial revision.
 * 
 *    Rev 1.11   06 Jun 1996 16:21:46   JHUCKINS
 * Added m_FileObject data member to dereference after calls to 
 * GetDeviceObjectPointer()
 * 
 *    Rev 1.10   06 Jun 1996 15:57:14   JHUCKINS
 * Added GetDeviceObjectPointer()
 * 
 *    Rev 1.9   03 Jun 1996 07:47:06   JHUCKINS
 * Added Author to file header
 * 
 *    Rev 1.8   31 May 1996 15:50:14   JDELLIOT
 * added robustness features (ASSERT, TRACE, validation of params)
 * cleanup
 * restructure of some interfaces
 * 
 *    Rev 1.7   17 May 1996 15:34:02   JHUCKINS
 * Added GetAttachedDeviceStackSize()
 * 
 *    Rev 1.6   16 May 1996 13:49:48   JHUCKINS
 * Resolve pure virtual OnInterrupt() from CDeviceObject
 * 
 *    Rev 1.5   16 May 1996 13:44:44   JHUCKINS
 * Resolved pure virtual GetDeviceDescription() from CDeviceObject
 * 
 *    Rev 1.4   16 May 1996 13:22:42   JHUCKINS
 * Added CDriverObject* argument to constructor
 * 
 *    Rev 1.3   08 Apr 1996 08:51:28   JHUCKINS
 * Added IN and OUT for formal parameters
 * 
 *    Rev 1.2   27 Mar 1996 14:13:28   JDELLIOT
 * added PVCS keywords and support for warning level 4
 *
 */

